package be.iminds.ilabt.jfed.ssh_terminal_tool.ssh_key_info;

import be.iminds.ilabt.jfed.lowlevel.ssh_key_info.SshKeyInfo;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.ssh_terminal_tool.putty.PuTTYPrivateKeyFile;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.PermanentFile;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.TmpFile;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.security.PrivateKey;
import java.security.PublicKey;

/**
 * PreferencesSshKeyInfo
 * <p/>
 * this normally nevers needs the private key, so it doesn't return it
 */
public class PreferencesPuttySshKeyInfo implements SshKeyInfo {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(PreferencesPuttySshKeyInfo.class);

    private final PuTTYPrivateKeyFile ppk;
    private File puttyKeyFile;

    private PublicKey publicKey;
    private PrivateKey privateKey;

    private final boolean complete;


    public PreferencesPuttySshKeyInfo(JFedPreferences jFedPreferences,
                                      JFedPreferences.PreferenceKey overridePref,
                                      JFedPreferences.PreferenceKey puttyKeyFilePref) {
        boolean overrideVal = jFedPreferences.getBoolean(overridePref);
        assert overrideVal;

        puttyKeyFile = jFedPreferences.getFile(puttyKeyFilePref);

        PuTTYPrivateKeyFile ppk = null;
        boolean complete = false;

        try {
            ppk = PuTTYPrivateKeyFile.read(puttyKeyFile);
            publicKey = ppk.getPublicKey();
            if (!ppk.isEncrypted())
                privateKey = ppk.getPrivateKey();
            else
                privateKey = null;

            assert publicKey != null;
            //all complete
            complete = true;
        } catch (IOException e) {
            LOG.error("Failed to read putty key file: " + e.getMessage(), e);
        }
        this.ppk = ppk;
        this.complete = complete;
    }

    @Override
    public TmpFile getPrivateKeyFile() {
        return null;
    }


    @Override
    public TmpFile getPuttyKeyFile() {
        return new PermanentFile(puttyKeyFile);
    }

    @Override
    public TmpFile getUnencryptedPrivateKeyFile() {
        return null;
    }

    @Override
    public TmpFile getUnlockedPuttyKeyFile() {
        return null;
    }

//    @Override
//    public void release() {
//    }

    @Override
    public boolean hasUnencryptedPrivateKey() {
        return false;
    }

    @Override
    public PublicKey getPublicKey() {
        return publicKey;
    }

    @Override
    public PrivateKey getPrivateKey() {
        return privateKey;
    }

    public boolean isComplete() {
        return complete;
    }

    public PuTTYPrivateKeyFile getPPK() {
        return ppk;
    }
}
