package be.iminds.ilabt.jfed.ui.javafx.am_list_gui;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import javafx.fxml.FXML;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;

/**
 * FXML Controller class
 *
 * @author wim
 */
public class AddAuthorityDialogController  {
    private static org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(AddAuthorityDialogController.class);

    @FXML private BorderPane pane;
    @FXML private TextField authorityUrnField;
    private Stage stage;

    //limit visibility of constructor
    AddAuthorityDialogController(){}

    /**
     * Initializes the controller class.
     */
    @FXML
    private void initialize() {
        String validatorCss = getClass().getResource("validation.css").toExternalForm();
        pane.getStylesheets().add(validatorCss);

        authorityUrnField.textProperty().addListener((observableValue, s, s1) -> validate());
    }

    public void setStage(Stage stage) {
        this.stage = stage;
    }

    public void cancel() {
        authorityUrnField.setText("");
        stage.close();
    }

    public void validate() {
        authorityUrnField.getStyleClass().removeAll("validation-error", "validation-warning");
        try {
            SfaAuthority.urnPartFromUrn(authorityUrnField.getText());
        } catch (JFedException e) {
            authorityUrnField.getStyleClass().add("validation-error");
        }
    }

    public void create() {
        authorityUrnField.getStyleClass().removeAll("validation-error", "validation-warning");
        try {
            SfaAuthority.urnPartFromUrn(authorityUrnField.getText());
            stage.close();
        } catch (JFedException e) {
            LOG.info("Invalid Urn \"" + authorityUrnField.getText() + "\"", e);
            authorityUrnField.getStyleClass().add("validation-error");
        }
    }

    public String getAuthorityUrn() {
        if (authorityUrnField.getText().equals("")) return null;
        return authorityUrnField.getText();
    }

}
