package be.iminds.ilabt.jfed.util;

import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.ListView;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.CheckBoxListCell;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.util.StringConverter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * CheckboxList
 * <p>
 * You can add a field to add items, but that only works when the type is String
 * <p>
 * TODO: add StringConverter support (both directions used if add is active)
 */
public class CheckboxList<T> extends VBox {
    @FXML
    protected ListView<SelectableImmutableString> checkboxes;
    @FXML
    protected TextField addTextField;
    @FXML
    protected HBox addBox;

    @FXML
    protected SimpleBooleanProperty canAdd = new SimpleBooleanProperty(false);

    protected final ObservableList<SelectableImmutableString> selectableOptions = FXCollections.observableArrayList();
    private final ObservableList<SelectableImmutableString> selectedOptionsList = FXCollections.observableArrayList();

    private class SelectableImmutableString {
        private BooleanProperty selected;
        private final String text;
        private final T object;

        private SelectableImmutableString(T object, String text, boolean selected) {
            this.object = object;
            this.text = text;
            this.selected = new SimpleBooleanProperty(selected);
            this.selected.addListener((observableValue, oldValue, newValue) -> {
                if (selectedOptionsList != null) {
                    if (!newValue)
                        selectedOptionsList.removeAll(SelectableImmutableString.this);
                    else {
                        assert !selectedOptionsList.contains(SelectableImmutableString.this);
                        selectedOptionsList.add(SelectableImmutableString.this);
                    }
                }
            });
            if (selected)
                selectedOptionsList.add(this);
        }

        public boolean getSelected() {
            return selected.get();
        }

        public BooleanProperty selectedProperty() {
            return selected;
        }

        public T getObject() {
            return object;
        }

        public String getText() {
            return text;
        }
    }

    public CheckboxList() {
        FXMLUtil.injectFXML(this);

        assert checkboxes != null;

        checkboxes.setCellFactory(list -> {
                    CheckBoxListCell<SelectableImmutableString> cb = new CheckBoxListCell<>();
                    cb.setConverter(new StringConverter<SelectableImmutableString>() {
                        @Override
                        public String toString(SelectableImmutableString selectableImmutableString) {
                            return selectableImmutableString.getText();
                        }

                        @Override
                        public SelectableImmutableString fromString(String s) {
                            throw new RuntimeException("This direction doesn't work and is not needed");
                        }
                    });
                    cb.setSelectedStateCallback(SelectableImmutableString::selectedProperty);
                    return cb;
                }
        );

        addBox.visibleProperty().bind(canAdd);
        addBox.managedProperty().bind(canAdd);

        checkboxes.setItems(this.selectableOptions);

    }

    public void setOptions(List<T> options) {
        setAllOptions(options);
    }

    public void clear() {
        setAllOptions(new ArrayList<>());
    }

    public void setSelectedOptions(List<T> selectedOptions) {
        selectedOptionsList.clear();
        for (SelectableImmutableString sis : selectableOptions) {
            sis.selectedProperty().setValue(selectedOptions.contains(sis.getObject()));
        }
    }

    public List<T> getSelectedItems() {
        List<T> selectedItems = new ArrayList<>();
        for (SelectableImmutableString sms : selectedOptionsList) {
            selectedItems.add(sms.getObject());
        }
        return Collections.unmodifiableList(selectedItems);
    }

    protected void setAllOptions(List<T> options) {
        selectableOptions.clear();
        selectedOptionsList.clear();
        for (T option : options) {
            SelectableImmutableString selMutStr = new SelectableImmutableString(option, option.toString(), false);
            selectableOptions.add(selMutStr);
        }
        selectedOptionsList.clear();
    }

    public ObservableList<SelectableImmutableString> selectableOptionsProperty() {
        return selectableOptions;
    }

    public ObservableList<SelectableImmutableString> selectedOptionsListProperty() {
        return selectedOptionsList;
    }

    public void setCanAdd(boolean canAdd) {
        this.canAdd.set(canAdd);
    }

    public boolean getCanAdd() {
        return canAdd.get();
    }

    public SimpleBooleanProperty canAddProperty() {
        return canAdd;
    }

    public void add() {
        //Add only works when type is String !
        SelectableImmutableString toAdd =
                new SelectableImmutableString((T) addTextField.getText(), addTextField.getText(), true);
        selectableOptions.add(toAdd);
    }
}
