package be.iminds.ilabt.jfed.experimenter_gui;

import be.iminds.ilabt.jfed.BaseGuiModule;
import be.iminds.ilabt.jfed.UncaughtGuiExceptionLogger;
import be.iminds.ilabt.jfed.experimenter_gui.debug.LogViewer;
import be.iminds.ilabt.jfed.experimenter_gui.ext_requests.EnvironmentParameterHandler;
import be.iminds.ilabt.jfed.experimenter_gui.ext_requests.ExternalUserRequestHandler;
import be.iminds.ilabt.jfed.experimenter_gui.ext_requests.FirstLoginParametersHandler;
import be.iminds.ilabt.jfed.experimenter_gui.ext_requests.JFedEnvironment;
import be.iminds.ilabt.jfed.experimenter_gui.login.URLLogin;
import be.iminds.ilabt.jfed.experimenter_gui.login.UserLoginController;
import be.iminds.ilabt.jfed.experimenter_gui.preferences.PreferencesDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.experimenter_gui.ui.AcceleratorService;
import be.iminds.ilabt.jfed.experimenter_gui.util.MacOpenURIHandler;
import be.iminds.ilabt.jfed.experimenter_gui.util.URIUtils;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.StageUtils;
import be.iminds.ilabt.jfed.highlevel.HighLevelModule;
import be.iminds.ilabt.jfed.highlevel.LowLevelModule;
import be.iminds.ilabt.jfed.highlevel.UserLoginModelModule;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.logging.JFedLoggerManager;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.ssh_key_info.SshKeyInfo;
import be.iminds.ilabt.jfed.lowlevel.userloginmodel.InvalidLoginException;
import be.iminds.ilabt.jfed.lowlevel.userloginmodel.UserLoginModelManager;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.preferences.JFedPreferencesApplier;
import be.iminds.ilabt.jfed.ssh_terminal_tool.putty.PageantHelper;
import be.iminds.ilabt.jfed.ssh_terminal_tool.ssh_key_info.UserSshKeyInfo;
import be.iminds.ilabt.jfed.ssh_terminal_tool.unix.SshAgentHelper;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.ui.javafx.util.ExtraInfoCallBackGUIs;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.JFedUtils;
import be.iminds.ilabt.jfed.util.OSDetector;
import com.cathive.fx.guice.GuiceApplication;
import com.cathive.fx.guice.GuiceFXMLLoader;
import com.google.inject.Module;
import javafx.application.Platform;
import javafx.collections.ObservableMap;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyCodeCombination;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Provider;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * User: twalcari
 * Date: 10/31/13
 * Time: 6:17 PM
 */
public class Launch extends GuiceApplication implements Provider<Stage> {
    private static final int MINIMUM_JAVA_UPDATE_VERSION = 60;

    /* solution for MAC OS HeadlessException bug. See https://ibcn-jira.intec.ugent.be/browse/FEDIBBTDEV-239 */
    static {
        System.setProperty("java.awt.headless", "false");
    }

    private static final Logger LOG = LoggerFactory.getLogger(Launch.class);
    @Inject
    private JFedLoggerManager loggerManager;
    @Inject
    private JFedGuiPreferences jFedPreferences;
    @Inject
    private JFedPreferencesApplier jFedPreferencesApplier;
    @Inject
    private UserLoginModelManager userLoginModelManager;
    @SuppressWarnings("MismatchedQueryAndUpdateOfCollection")
    @Inject
    private ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers;
    @Inject
    private TaskThread tt;
    @Inject
    private GuiceFXMLLoader loader;
    @Inject
    private PageantHelper pageantHelper;

    @Inject
    private PreferencesDialogFactory preferencesDialogFactory;

    private Stage primaryStage;
    @Inject
    private AcceleratorService acceleratorService;

    @Inject
    private FirstLoginParametersHandler firstLoginParametersHandler;

    @Inject
    private ExternalUserRequestHandler externalUserRequestHandler;

    @Inject
    private EnvironmentParameterHandler environmentParameterHandler;

    @Inject
    private Provider<MacOpenURIHandler> macOpenURIHandlerProvider;

    @Inject
    private Provider<LogViewer> logviewerProvider;

    @Inject
    private JFedEnvironment environment;

    //keep reference to prevent it from being Garbage-collected
    @SuppressWarnings({"unused", "FieldCanBeLocal"})
    private MacOpenURIHandler macOpenURIHandler = null;

    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(final Stage stage) {
        this.primaryStage = stage;

        loggerManager.updateLoggers();

        //catch all exceptions which aren't caught anywhere else
        Thread.setDefaultUncaughtExceptionHandler(UncaughtGuiExceptionLogger.getInstance(stage));

        if (!isRecentJavaVersion()) {
            JFDialogs.create()
                    .owner(stage)
                    .message("You are currently using a Java-version with known bugs, which can result in unstable behavior of jFed. Please update your Java installation to the latest version.")
                    .title("Outdated Java installation detected")
                    .masthead("Outdated Java installation detected")
                    .showWarning();
        }

        try {
            //check for filesystem access
            if (JFedUtils.getUserDataDirectory() == null) {

                JFDialogs.create()
                        .owner(stage)
                        .message("Please verify your security settings and try again.\n\n" +
                                "If you are trying to run jFed through Safari on OS X, you need to change your Safari security settings.\n" +
                                "(cfr. http://doc.fed4fire.eu/firstexperiment.html#start-up-jfed)")
                        .masthead("jFed was unable to access the filesystem.")
                        .title("Fatal Error").showError();

                exit();
                return;
            }

            //register Mac URI handler when on Mac
            if (OSDetector.findOS() == OSDetector.OS.MAC) {
                macOpenURIHandler = macOpenURIHandlerProvider.get();
            }

            //process startup parameters on all platforms
            Map<String, String> parameters = getAllParameters();
            externalUserRequestHandler.handleParameters(parameters);
            environmentParameterHandler.handleParameters(parameters);

            ExtraInfoCallBackGUIs.prepareExtraInfoCallback(stage);

            FirstLoginParametersHandler.LoginResult loginResult =
                    firstLoginParametersHandler.handleParameters(parameters);


            if (!loginResult.isLoginDefined() && environment.getLoginUrl() != null) {
                LOG.info("Got login url '{}'. Showing dialog.", environment.getLoginUrl());
                URLLogin urlLogin = new URLLogin(environment.getLoginUrl(), logviewerProvider);
                String certificate = urlLogin.showDialog();

                if (certificate != null) {
                    try {
                        userLoginModelManager.setUserLoginModelType(UserLoginModelManager.UserLoginModelType.KEY_CERT_INTERNAL_INFO);
                        userLoginModelManager.getKeyCertUserLoginModel().setKeyCertPemString(certificate);
                        loginResult.setLoginDefined(true);
                    } catch (InvalidLoginException ex) {
                        LOG.error("Could not login with content of login-certificate:\n{}", certificate, ex);
                    }
                    LOG.info("User successfully fetched certificate");
                } else {
                    LOG.warn("User did not successfully fetch certificate");
                }

                if (loginResult.isLoginDefined()) {
                    if (!userLoginModelManager.getKeyCertUserLoginModel().isPasswordRequired()) {
                        userLoginModelManager.login();
                        loginResult.setLoggedIn(true);
                    } else {
                        //certificate is password protected, so we can save it locally!
                        File certificateFile = URLLogin.saveCertificate(certificate);
                        try {
                            userLoginModelManager.setUserLoginModelType(UserLoginModelManager.UserLoginModelType.KEY_CERT_INTERNAL_INFO);
                            userLoginModelManager.getKeyCertUserLoginModel().setKeyCertPemFile(certificateFile);
                            loginResult.setLoginDefined(true);
                        } catch (InvalidLoginException ex) {
                            LOG.error("Could not login with content of saved login-certificate {}",
                                    certificateFile != null ? certificateFile.getAbsolutePath() : "null", ex);
                        }

                    }
                }
            }

            if (!loginResult.isLoggedIn()) {
                if (!loginResult.isLoginDefined()) {
                    userLoginModelManager.load();
                }

                //does not wait! UserLoginControllerListener can be used to see if login window is closed
                showUserLogin(stage);
            } else {
                onSuccessfulLogin(stage);
            }
        } catch (Throwable ex) {
            //catch Throwable is allowed here, as we throw it afterwards
            LOG.error("Fatal error in jFed Experimenter Main Thread: " + ex.getMessage(), ex);
            JFDialogs.create()
                    .owner(stage)
                    .message("jFed experimenter will terminate.")
                    .masthead("A fatal error occurred!")
                    .title("Fatal Error")
                    .showException(ex);

            throw ex;
        }
    }

    public Map<String, String> getAllParameters() {

        Map<String, String> parameters = new HashMap<>();

        parameters.putAll(getParameters().getNamed());

        for (String unnamedParam : getParameters().getUnnamed()) {
            Map<String, String> uriResult = URIUtils.processJFedUri(unnamedParam);

            if (uriResult != null) {
                parameters.putAll(uriResult);
            }
        }
        return parameters;

    }

    private void onSuccessfulLogin(Stage stage) {
        try {
            //show information dialog about public/private keys on first run of jFed

            if (jFedPreferences.getInt(GuiPreferenceKey.PREF_FIRSTRUN_VERSION) < JFedPreferences.CURRENT_FIRSTRUN_VERSION) {

                if (OSDetector.findOS() == OSDetector.OS.WIN) {


                    JFDialogs.create()
                            .owner(stage)
                            .message("The default preferences should be okay. If you have trouble running experiments or connecting to nodes, consider to turn on 'Proxy' in the jFed Proxy preferences.\n\n" +
                                    "Next you will see the preferences window, you can always revisit this by clicking the preferences button.\n\n" +
                                    "Please verify that jFed has found the PuTTY SSH software (you should see 4 green checkmarks). If not install first PuTTY (full suite) or point jFed to the right PuTTY install directory.")
                            .masthead("Initial configuration")
                            .title("Welcome to the jFed Experimenter Tool!")
                            .showInformation();


                } else {

                    JFDialogs.create()
                            .owner(stage)
                            .message("The default preferences should be okay. If you have trouble running experiments or connecting to nodes, consider to turn on 'Proxy' in the jFed Proxy preferences.\n\n" +
                                    "Next you will see the preferences window, you can always revisit this by clicking the preferences button.")
                            .masthead("Initial configuration")
                            .title("Welcome to the jFed Experimenter Tool!")
                            .showInformation();

                }

                preferencesDialogFactory.showDialog();

                jFedPreferences.setInt(GuiPreferenceKey.PREF_FIRSTRUN_VERSION, JFedPreferences.CURRENT_FIRSTRUN_VERSION);
            }

            jFedPreferencesApplier.applyPreferences();

            showExperimenterGui(stage);
        } catch (Throwable ex) {
            //catch Throwable is allowed here, as we throw it afterwards
            LOG.error("Fatal error in jFed Experimenter Main Thread: " + ex.getMessage(), ex);
            JFDialogs.create()
                    .owner(stage)
                    .message("jFed experimenter will terminate.")
                    .masthead("A fatal error occurred!")
                    .title("Fatal Error")
                    .showException(ex);

            throw ex;
        }
    }

    private void registerKeysWithPageant() {
        if (OSDetector.findOS() != OSDetector.OS.WIN)
            return;

        if (jFedPreferences.getBoolean(GuiPreferenceKey.PREF_SSHAGENT_USE, false)) {
            List<SshKeyInfo> keys = new ArrayList<>();

            SshKeyInfo sshKeyInfo = jFedPreferences.getOverriddenSshKeyInfo();
            if (sshKeyInfo == null && userLoginModelManager.isUserLoggedIn()) {
                GeniUser geniUser = userLoginModelManager.getLoggedInGeniUser();
                sshKeyInfo = new UserSshKeyInfo(geniUser);
            }
            keys.add(sshKeyInfo);

            if (!jFedPreferences.getString(GuiPreferenceKey.PREF_SSHPROXY_USE_FOR_SSH).equalsIgnoreCase("NEVER")) {
                SshKeyInfo proxySshKeyInfo = jFedPreferences.getOverriddenProxySshKeyInfo();
                if (proxySshKeyInfo == null && userLoginModelManager.isUserLoggedIn()) {
                    GeniUser geniUser = userLoginModelManager.getLoggedInGeniUser();
                    proxySshKeyInfo = new UserSshKeyInfo(geniUser);
                }
                keys.add(proxySshKeyInfo);
            }

            pageantHelper.registerKeys(keys);
        }
    }

    /**
     * doesn't wait
     */
    public void showUserLogin(final Stage parentStage) {

        UserLoginController.UserLoginControllerListener userLoginControllerListener = successfulLogin -> {
            if (successfulLogin)
                onSuccessfulLogin(parentStage);
            else {
                exit();
            }
        };

        try {
            URL location = UserLoginController.class.getResource("UserLogin.fxml");

            GuiceFXMLLoader.Result result = loader.load(location);

            final UserLoginController userLoginController = result.getController();
            userLoginController.addListener(userLoginControllerListener);

            Stage userLoginStage = new Stage();
            userLoginController.setupStage(userLoginStage);

            userLoginStage.setTitle("jFed login");
            userLoginStage.setResizable(false);
            final Scene scene = new Scene(result.getRoot());
            scene.getAccelerators().put(
                    new KeyCodeCombination(KeyCode.F10, KeyCodeCombination.CONTROL_DOWN),
                    () -> logviewerProvider.get().show()
            );


            userLoginStage.setScene(scene);
            userLoginStage.getIcons().addAll(new Image(UserLoginController.class.getResource("/images/jfed-logo_16.png").toExternalForm())
                    , new Image(UserLoginController.class.getResource("/images/jfed-logo_32.png").toExternalForm())
                    , new Image(UserLoginController.class.getResource("/images/jfed-logo_64.png").toExternalForm()));


            //don't wait!
            userLoginStage.show();
        } catch (Exception e) {
            throw new RuntimeException("Something went wrong showing the User Login Screen: " + e.getMessage(), e);
        }
    }


    private boolean isExperimentActive(Experiment experiment) {
        if (experiment.getSlice() == null)
            return false;

        for (Sliver sliver : experiment.getSlice().getSlivers()) {
            switch (sliver.getStatus().getGlobalStatus()) {
                case READY:
                case UNKNOWN:
                case CHANGING:
                    return true;
            }
        }
        return false;
    }


    public void showExperimenterGui(Stage stage) {
        try {
            GuiceFXMLLoader.Result result = loader.load(getClass().getResource("ExperimenterGUI.fxml"));
            Scene scene = new Scene(result.getRoot());

            ExperimenterGUI experimenterGUI = result.getController();

            stage.setScene(scene);
            stage.setMinHeight(600);
            stage.setMinWidth(800);

            stage.setHeight(600);
            stage.setWidth(1024);

            stage.setTitle("jFed Experimenter Toolkit");
            StageUtils.setJFedStageIcons(stage);

            stage.setOnCloseRequest(
                    event -> {
                        if (!experimenterGUI.onStageCloseRequest()) {
                            event.consume();
                        } else {
                            exit();
                        }
                    });

            //Called just after the Window is shown.
            stage.setOnShown(windowEvent -> {
                registerKeysWithPageant();
                externalUserRequestHandler.activate();
            });

            acceleratorService.registerOnScene(scene);


            stage.show();
        } catch (IOException ex) {
            LOG.error("Error while loading ExperimenterGUI", ex);
            throw new RuntimeException("Error while loading ExperimenterGUI", ex);
        }
    }

    public void exit() {
        LOG.info("Close was requested. Trying to terminate");
        tt.requestStop();

        //stop any running ssh-agent
        SshAgentHelper.stopSshAgent();

        Platform.exit();
        System.exit(0);
    }

    @Override
    public void init(List<Module> modules) throws Exception {
        modules.add(new LowLevelModule());
        modules.add(new HighLevelModule());
        modules.add(new UserLoginModelModule());
        modules.add(new ExperimenterModule());
        modules.add(new BaseGuiModule());
    }

    @Override
    public Stage get() {
        return primaryStage;
    }

    public boolean isRecentJavaVersion() {
        String versionString = System.getProperty("java.version");

        if (versionString.startsWith("1.7")) {
            return false;
        } else if (versionString.startsWith("1.8")) {
            final int separationCharLocation = versionString.indexOf("_");
            if (separationCharLocation < 0) {
                LOG.warn("Unable to detect update version of Java in version-string {}", versionString);
                return true;
            }

            try {
                int updateVersion = Integer.parseInt(versionString.substring(separationCharLocation + 1));
                return updateVersion > MINIMUM_JAVA_UPDATE_VERSION;
            } catch (NumberFormatException ex) {
                LOG.warn("Could not determine Java update version in version-string {}", versionString);
                return true;
            }
        } else {
            LOG.warn("Could not determine major Java version in version-string {}", versionString);
            return true; //probably a future version: Java9 or higher
        }
    }
}
