package be.iminds.ilabt.jfed.experimenter_gui.canvas;

import be.iminds.ilabt.jfed.experimenter_gui.editor.ToolboxItem;
import javafx.beans.InvalidationListener;
import javafx.beans.Observable;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.StringProperty;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 10/18/13
 * Time: 5:16 PM
 */
public class CanvasNode extends BorderPane {
    private static final Logger LOG = LoggerFactory.getLogger(CanvasNode.class);
    protected final ExperimentCanvas experimentCanvas;

    protected final ImageView typeImageView;
    protected final Label nameLabel;
    protected final StackPane testbedPane;
    protected final ImageView testbedImageView;
    protected final Tooltip tooltip;

    protected final StringProperty tooltipText;
    protected final StringProperty text;
    protected final ObjectProperty<Image> image;
    protected final ObjectProperty<Color> color = new SimpleObjectProperty<>(Color.TRANSPARENT);
    protected final ObjectProperty<Image> logo;

    public CanvasNode(final ExperimentCanvas experimentCanvas, String text, Image image) {
        this.experimentCanvas = experimentCanvas;


        this.typeImageView = new ImageView(image);
        typeImageView.setFitWidth(25);
        typeImageView.setFitHeight(25);
        typeImageView.setPreserveRatio(true);
        typeImageView.getStyleClass().add("canvas-node-image");
        this.image = typeImageView.imageProperty();

        this.nameLabel = new Label(text);
        this.text = nameLabel.textProperty();

        this.testbedPane = new StackPane();
        testbedPane.setPrefWidth(30);

        //this.color = testbedPane.fillProperty();
        this.color.addListener(new InvalidationListener() {
            @Override
            public void invalidated(Observable observable) {
                testbedPane.setStyle(String.format("-fx-background-color: linear-gradient(from 0%% 0%% to 100%% 0%%, transparent 0%%, rgb(%s,%s,%s)  100%%)",
                        CanvasNode.this.color.get().getRed() * 255, CanvasNode.this.color.get().getGreen() * 255, CanvasNode.this.color.get().getBlue() * 255));
            }
        });

        testbedPane.getStyleClass().add("canvas-node-shape");
        BorderPane.setAlignment(testbedPane, Pos.CENTER);

        testbedImageView = new ImageView();
        testbedImageView.setFitHeight(25);
        testbedImageView.setFitWidth(25);
        testbedPane.getChildren().add(testbedImageView);
        this.logo = testbedImageView.imageProperty();


        this.tooltip = new Tooltip();
        tooltipText = tooltip.textProperty();
        Tooltip.install(testbedPane, tooltip);

        getStylesheets().add(CanvasNode.class.getResource("canvas.css").toExternalForm());
        getStyleClass().add("canvas-node");
        nameLabel.getStyleClass().add("canvas-node-text");

        setMinSize(100, 30);

        setLeft(typeImageView);
        setCenter(nameLabel);
        setRight(testbedPane);

        experimentCanvas.getSelectionProvider().selectedObjectProperty().addListener(
                ((observable, oldValue, newValue) -> setFocused(this == newValue)));
    }

    public CanvasNode(CanvasNode item) {
        this(item.experimentCanvas, item.nameLabel.getText(), item.getTypeImageView().getImage());
    }

    public CanvasNode(ToolboxItem item, ExperimentCanvas experimentCanvas) {
        this(experimentCanvas, item.getText(), item.getImageView().getImage());
    }

    public ImageView getTypeImageView() {
        return typeImageView;
    }

    public String getText() {
        return text.get();
    }

    public StringProperty textProperty() {
        return text;
    }

    public Image getImage() {
        return image.get();
    }

    public ObjectProperty<Image> imageProperty() {
        return image;
    }

    public void setText(String text) {
        this.text.set(text);
    }

    public void setImage(Image image) {
        this.image.set(image);
    }
}
