package be.iminds.ilabt.jfed.experimenter_gui.canvas;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.util.ImageSize;
import be.iminds.ilabt.jfed.experimenter_gui.util.ImageUtil;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.scene.image.Image;
import javafx.scene.paint.Color;

/**
 * User: twalcari
 * Date: 4/24/2015
 * Time: 10:50
 */
public class TestbedAwareCanvasNode extends CanvasNode {

    private final AuthorityList authorityList;
    private final JFedConfiguration config;

    protected final ObjectProperty<GeniUrn> componentManagerId = new SimpleObjectProperty<>();

    {
        componentManagerId.addListener(obj -> updateTestbed());
    }

    public TestbedAwareCanvasNode(ExperimentCanvas experimentCanvas, String text, Image image,
                                  AuthorityList authorityList, JFedConfiguration config) {
        super(experimentCanvas, text, image);
        this.authorityList = authorityList;
        this.config = config;
    }


    private void updateTestbed() {

        if(componentManagerId.get() != null) {

            AuthorityInfo authorityInfo =
                    authorityList.getByUrnExact(componentManagerId.get()); //not ALLOW_TOPLEVEL here!   (purpose would be REQUEST)
            if (authorityInfo != null) {

                tooltipText.setValue(authorityInfo.getSfaAuthority().getHrn());
                JFedConfiguration.TestbedDescription testbedDescription =
                        config.getTestbedDescription(componentManagerId.get());

                if (testbedDescription != null) {
                    color.setValue(testbedDescription.getColor());
                    if (testbedDescription.getLogo() != null)
                        logo.setValue(ImageUtil.getTestbedDescriptionImage(testbedDescription, ImageSize.SMALL));
                    else
                        logo.setValue(null);
                } else {
                    color.setValue(Color.TRANSPARENT);
                    logo.setValue(null);
                }
            } else {
                color.setValue(Color.TRANSPARENT);
            }
        }else{
            logo.setValue(null);
            color.setValue(Color.TRANSPARENT);
        }
    }
}
