package be.iminds.ilabt.jfed.experimenter_gui.connectivity_tester;

import be.iminds.ilabt.jfed.connectivity_tester.ConnectivityTest;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.concurrent.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A ConnectivityTestTask is an instance of an {@link javafx.concurrent.Task} that actually executes a
 * {@link be.iminds.ilabt.jfed.connectivity_tester.ConnectivityTest}
 *
 * User: twalcari
 * Date: 1/6/14
 * Time: 11:57 AM
 */
public class ConnectivityTestTask extends Task<TaskStatusIndicator.Status> {
    private static final Logger LOG = LoggerFactory.getLogger(ConnectivityTestTask.class);

    public static final double TASK_STARTED = 0.5, TASK_ENDED = 1;
    private static final double END_STATUS = 1;

    private final ConnectivityTest test;
    private ObjectProperty<ConnectivityTest.Status> status = new SimpleObjectProperty<>(null);
    private ConnectivityTest.ConnectivityTestResult result = null;

    /**
     * Resulting status of the test task
     * @return
     */
    public ConnectivityTest.Status getStatus() {
        return status.get();
    }

    public ObjectProperty<ConnectivityTest.Status> statusProperty() {
        return status;
    }

    public ConnectivityTestTask(ConnectivityTest test) {
        this.test = test;
    }

    @Override
    protected TaskStatusIndicator.Status call() throws Exception {
        updateProgress(TASK_STARTED, END_STATUS);

        try {
            result = test.call();
        } catch (Exception | AssertionError t) {
            LOG.error("AssertionError in ConnectivityTest: " + t.getMessage(), t);
            status.set(ConnectivityTest.Status.FAILED);
            updateProgress(TASK_ENDED, END_STATUS);
            return TaskStatusIndicator.Status.FAILED;
        }

        status.set(result.getStatus());

        updateProgress(TASK_ENDED, END_STATUS);

        switch (result.getStatus()) {
            case FAILED:
                return TaskStatusIndicator.Status.FAILED;
            case WARNING:
                return TaskStatusIndicator.Status.WARNING;
            case SUCCEEDED:
                return TaskStatusIndicator.Status.SUCCESS;
            case SKIPPED:
                return TaskStatusIndicator.Status.INACTIVE;
            default:
                throw new IllegalStateException();
        }

    }

    /**
     * The test associated with this task
     * @return
     */
    public ConnectivityTest getTest() {
        return test;
    }

    /**
     * The result of the test.
     * @return
     */
    public ConnectivityTest.ConnectivityTestResult getResult() {
        return result;
    }
}
