package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.tasks.FetchAllSlicesInfoTask;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.RecoverSliceTask;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.TasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar.TaskService;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.OneTrueInListBinding;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.beans.InvalidationListener;
import javafx.beans.Observable;
import javafx.beans.binding.BooleanBinding;
import javafx.beans.binding.DoubleBinding;
import javafx.beans.property.SimpleStringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.transformation.FilteredList;
import javafx.collections.transformation.SortedList;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.geometry.Pos;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import javafx.util.Callback;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.control.action.Action;
import org.controlsfx.dialog.Dialog;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Provider;
import java.time.Duration;
import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Comparator;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 12/5/13
 * Time: 9:21 PM
 */
public class RecoverSliceDialog extends VBox {
    private static final Logger LOG = LoggerFactory.getLogger(RecoverSliceDialog.class);
    private static final DateTimeFormatter EXPIRATION_TIME_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");

    private static final String ALL_PROJECTS = "All projects";

    @FXML
    private TextField nameFilterTextField;
    @FXML
    private ComboBox<String> projectFilterComboBox;

    @FXML
    private TableView<Slice> slicesTableView;

    @FXML
    private TableColumn<Slice, Boolean> sliceHasSliversTableColumn;
    @FXML
    private TableColumn<Slice, String> sliceNameTableColumn;
    @FXML
    private TableColumn<Slice, String> subAuthTableColumn;
    @FXML
    private TableColumn<Slice, Instant> expirationTableColumn;
    @FXML
    private HBox initializingBox;

    @FXML
    private ProgressIndicator refreshProgressIndicator;
    @FXML
    private Label refreshLabel;

    private final EasyModel easyModel;
    private final HighLevelController hlc;
    private final TasksFactory tasksFactory;

    private final EventHandler<MouseEvent> mouseEventEventHandler = mouseEvent -> {
        if (mouseEvent.getClickCount() == 2) {
            recoverExperiment();
        }

    };

    private final Dialog dialog;

    private final TaskService taskService;
    private final Provider<Stage> parentStageProvider;
    private final BooleanBinding atLeastOneSliceWithExpirationDate;

    @Inject
    RecoverSliceDialog(EasyModel easyModel, HighLevelController hlc, TasksFactory tasksFactory, TaskService taskService, @Named("parentStage") Provider<Stage> parentStageProvider) {
        this.easyModel = easyModel;
        this.hlc = hlc;
        this.tasksFactory = tasksFactory;
        this.taskService = taskService;
        this.parentStageProvider = parentStageProvider;

        //initialize the Scene
        FXMLUtil.injectFXML(this);

        SortedList<Slice> sortedSlices =
                new SortedList<>(this.easyModel.getSlices(), Comparator.comparing(Slice::getHasActiveSlivers,
                        Comparator.nullsLast(Comparator.naturalOrder())
                      /*  ,(sliceHasSliversA, sliceHasSliversB) -> {
                            //always show slices with active slivers first
                            if (sliceHasSliversA == Boolean.TRUE && sliceHasSliversB != Boolean.TRUE)
                                return -1;
                            if (sliceHasSliversA != Boolean.TRUE && sliceHasSliversB == Boolean.TRUE)
                                return 1;

                            return 0;
                        }*/).thenComparing(Slice::getName, String.CASE_INSENSITIVE_ORDER));

        FilteredList<Slice> filteredAndSortedSlices = new FilteredList<>(sortedSlices);
        slicesTableView.setItems(filteredAndSortedSlices);

        sliceNameTableColumn.setCellValueFactory(new PropertyValueFactory<>("name"));
        sliceNameTableColumn.setCellFactory(sliceStringTableColumn -> new SliceStringTableCell());
        sliceHasSliversTableColumn.setCellValueFactory(t -> t.getValue().hasActiveSliversProperty());
        sliceHasSliversTableColumn.setCellFactory(new Callback<TableColumn<Slice, Boolean>, TableCell<Slice, Boolean>>() {
            @Override
            public TableCell<Slice, Boolean> call(TableColumn<Slice, Boolean> sliceStringTableColumn) {
                return new TableCell<Slice, Boolean>() {
                    {
                        setContentDisplay(ContentDisplay.GRAPHIC_ONLY);
                        setAlignment(Pos.CENTER);
                        setOnMouseClicked(mouseEventEventHandler);
                    }

                    @Override
                    protected void updateItem(Boolean hasActiveSlivers, boolean empty) {
                        super.updateItem(hasActiveSlivers, empty);
                        if (empty) {
                            setGraphic(null);
                        } else if (hasActiveSlivers == Boolean.FALSE) {
                            setGraphic(GlyphUtils.createGlyph(FontAwesome.Glyph.TIMES, Color.DARKGRAY));
                        } else if (hasActiveSlivers == Boolean.TRUE) {
                            setGraphic(GlyphUtils.createGlyph(FontAwesome.Glyph.CHECK_SQUARE, Color.GREEN));
                        } else if (hasActiveSlivers == null) {
                            setGraphic(GlyphUtils.createGlyph(FontAwesome.Glyph.QUESTION_CIRCLE, Color.BLACK));
                        }
                    }
                };
            }
        });
        subAuthTableColumn.setCellValueFactory(t -> new SimpleStringProperty(t.getValue().getSubAuthorityName()));
        subAuthTableColumn.setCellFactory(param -> new SliceStringTableCell());

        expirationTableColumn.setCellValueFactory(param -> param.getValue().expirationDateProperty());
        expirationTableColumn.setCellFactory(new Callback<TableColumn<Slice, Instant>, TableCell<Slice, Instant>>() {
            @Override
            public TableCell<Slice, Instant> call(TableColumn<Slice, Instant> param) {
                return new TableCell<Slice, Instant>() {
                    @Override
                    protected void updateItem(Instant item, boolean empty) {
                        super.updateItem(item, empty);

                        final Slice slice = (Slice) getTableRow().getItem();

                        if (slice != null) {
                            if (item != null) {
                                setGraphic(null);
                                setText(EXPIRATION_TIME_FORMATTER.format(item.atZone(ZoneId.systemDefault())));

                                Duration duration = Duration.between(Instant.now(), item);

                                if (duration.isNegative()) {
                                    setTextFill(Color.RED);
                                } else if (duration.toHours() < 12) {
                                    setTextFill(Color.ORANGERED);
                                } else if (duration.toHours() < 24) {
                                    setTextFill(Color.ORANGE);
                                } else if (duration.toDays() < 7) {
                                    setTextFill(Color.GREEN);
                                } else {
                                    setTextFill(Color.DARKGREEN);
                                }

                                LOG.debug("Setting expiration time of {} to {}", slice.getName(), item);
                            } else {
                                final ProgressIndicator progressIndicator = new ProgressIndicator(-1);
                                progressIndicator.setPrefSize(20, 20);
                                setGraphic(progressIndicator);
                                setText(null);

                                LOG.debug("No expiration time available for {}", slice.getName());
                            }
                        } else {
                            setGraphic(null);
                            setText(null);
                        }
                    }
                };
            }
        });

        //check if we have expiration dates to show
        atLeastOneSliceWithExpirationDate = new OneTrueInListBinding<>(filteredAndSortedSlices,
                slice -> slice.expirationDateProperty().isNotNull());

        DoubleBinding safeWidth = slicesTableView.widthProperty().subtract(3);
        sliceHasSliversTableColumn.prefWidthProperty().bind(safeWidth.multiply(0.1));
        sliceNameTableColumn.prefWidthProperty().bind(safeWidth.multiply(0.4));
        subAuthTableColumn.prefWidthProperty().bind(safeWidth.multiply(0.15));
        expirationTableColumn.prefWidthProperty().bind(safeWidth.multiply(0.35));

        //apply filters when necessary

        InvalidationListener filterListener = observable -> {

            String nameFilterText = nameFilterTextField.getText().trim().toLowerCase();
            String selectedProject = projectFilterComboBox.getSelectionModel().getSelectedItem();

            filteredAndSortedSlices.setPredicate(slice ->
                    (nameFilterText.isEmpty()
                            || slice.getName().toLowerCase().contains(nameFilterText))
                            &&
                            (selectedProject == null
                                    || ALL_PROJECTS.equals(selectedProject)
                                    || selectedProject.equals(slice.getSubAuthorityName())));
        };


        nameFilterTextField.textProperty().addListener(filterListener);
        projectFilterComboBox.getSelectionModel().selectedItemProperty().addListener(filterListener);


        easyModel.getLoggedInUserSubAuthorityNames().addListener((InvalidationListener) obs -> {
            LOG.info("Subauthoritynames was updated!");
            updateProjectsList();
        });
        easyModel.getSlices().addListener((InvalidationListener) obs -> {
            LOG.info("Slices was updated! Now {} items", easyModel.getSlices().size());
            updateProjectsList();
        });

        updateProjectsList();

        //show loading progressview when initializing
        javafx.concurrent.Task loadingTask = new javafx.concurrent.Task<Void>() {
            Task listSlicesTask = hlc.getSlicesForUser();

            @Override
            protected Void call() throws Exception {
                while (listSlicesTask.isActive()) {
                    Thread.sleep(100);
                }

                //check if some resolveSlice-calls are still running
                for (Slice slice : RecoverSliceDialog.this.easyModel.getSlices()) {
                    while (hlc.getAggregatesForSlice(slice).isActive())
                        Thread.sleep(100);
                }
                return null;
            }

            @Override
            protected void succeeded() {
                initializingBox.setVisible(false);
            }
        };

        final Thread thread = new Thread(loadingTask);
        thread.setDaemon(true);
        thread.setName("Recover-Loading-Task");
        thread.start();

        dialog = new Dialog(parentStageProvider.get(), "Recover an existing experiment", false /* lightweight */);

        dialog.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.AMBULANCE, Color.GREEN));
        dialog.setMasthead("Select the experiment to recover");
        dialog.setContent(this);

        Action refreshAction = new DialogAction("Refresh & Fetch exp. dates", ButtonBar.ButtonType.OTHER,
                false, false, false, actionEvent -> refreshSlicesList());
        refreshAction.setGraphic(GlyphUtils.createSmallGlyph(FontAwesome.Glyph.REFRESH, Color.GREEN).useGradientEffect());

        Action startExperimentAction =
                new DialogAction("Recover Experiment", ButtonBar.ButtonType.OK_DONE, false, false, true,
                        actionEvent -> recoverExperiment());
        dialog.getActions().setAll(refreshAction, startExperimentAction);

    }

    private void updateProjectsList() {

        ObservableList<String> projects = FXCollections.observableArrayList();
        projects.add(ALL_PROJECTS);

        projects.addAll(easyModel.getLoggedInUserSubAuthorityNames().stream().sorted().collect(Collectors.toList()));

        projectFilterComboBox.setItems(projects);

        LOG.info("Updating project list with {} projects", projects.size());

        if (projectFilterComboBox.getSelectionModel().isEmpty()) {
            projectFilterComboBox.getSelectionModel().select(ALL_PROJECTS);
        }
    }

    public void showDialog() {
        dialog.show();
    }

    private void refreshSlicesList() {
        refreshProgressIndicator.visibleProperty().unbind();
        refreshLabel.textProperty().unbind();

        FetchAllSlicesInfoTask fetchAllSlicesInfoTask = tasksFactory.createFetchAllSlicesInfoTask();

        refreshProgressIndicator.visibleProperty().bind(fetchAllSlicesInfoTask.runningProperty());
        refreshLabel.textProperty().bind(fetchAllSlicesInfoTask.messageProperty());
        taskService.submitTask(fetchAllSlicesInfoTask);

    }

    public void recoverExperiment() {

        if (slicesTableView.getSelectionModel().isEmpty()) {
            JFDialogs.create()
                    .owner(this)
                    .message("Please select the experiment you wish to recover")
                    .masthead("No experiment selected")
                    .showError();
            return;
        }
        dialog.hide();

        final Slice recoverSlice = slicesTableView.getSelectionModel().getSelectedItem();

        RecoverSliceTask recoverSliceTask = tasksFactory.createRecoverSliceTask(recoverSlice);
        taskService.submitTask(recoverSliceTask);


    }

    private class SliceStringTableCell extends TableCell<Slice, String> implements InvalidationListener {

        Slice currentlyBoundSlice = null;

        {
            setOnMouseClicked(mouseEventEventHandler);
        }

        @Override
        protected void updateItem(String s, boolean b) {
            if (s != null) {
                setText(s);
            } else {
                setText("");
            }


            if (currentlyBoundSlice != null) {
                currentlyBoundSlice.hasActiveSliversProperty().removeListener(this);
                currentlyBoundSlice = null;
            }
            Slice slice = ((TableRow<Slice>) getTableRow()).getItem();
            if (slice != null) {
                slice.hasActiveSliversProperty().addListener(this);
                currentlyBoundSlice = slice;
            }
            updateColor();
        }

        @Override
        public void invalidated(Observable observable) {
            updateColor();
        }

        private void updateColor() {
            @SuppressWarnings("unchecked")
            Slice slice = ((TableRow<Slice>) getTableRow()).getItem();
            if (slice != null) {
                if (slice.getHasActiveSlivers() == Boolean.FALSE) {
                    setTextFill(Color.GRAY);
                } else {
                    setTextFill(Color.BLACK);
                }
            }
        }
    }
}
