package be.iminds.ilabt.jfed.experimenter_gui.editor.properties.map;

import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi.CMIManager;
import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi.ComponentManagerInfo;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import com.lynden.gmapsfx.ClusteredGoogleMapView;
import com.lynden.gmapsfx.MapComponentInitializedListener;
import com.lynden.gmapsfx.javascript.event.UIEventType;
import com.lynden.gmapsfx.javascript.object.*;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 1/8/2015
 * Time: 14:34
 */
public class TestbedsMapDialog implements MapComponentInitializedListener {
    private static final Logger LOG = LoggerFactory.getLogger(TestbedsMapDialog.class);

    private ClusteredGoogleMapView mapView;

    @FXML
    private BorderPane root;
    @FXML
    private Label testbedNameLabel;
    @FXML
    private Label testbedCountryLabel;
    @FXML
    private Button selectTestbedButton;

    private boolean mapViewIsInitialized = false;

    private ClusteredGoogleMap map;

    private CMIManager cmiManager;

    private final ObjectProperty<ComponentManagerInfo> selectedCmi = new SimpleObjectProperty<>();
    private ComponentManagerInfo result = null;

    @FXML
    private void initialize() {
        mapView = new ClusteredGoogleMapView(true);
        mapView.addMapInializedListener(this);

        root.setCenter(mapView);


        selectTestbedButton.disableProperty().bind(selectedCmi.isNull());
    }

    @Override
    public void mapInitialized() {
        mapViewIsInitialized = true;

        tryInitializeMap();


    }

    private void tryInitializeMap() {
        if (cmiManager == null) return;
        if (!mapViewIsInitialized) return;

        MapOptions mapOptions = new MapOptions();

        mapOptions.mapType(MapTypeIdEnum.ROADMAP)
                .center(new LatLong(47.56, 372))
                .overviewMapControl(true)
                .panControl(true)
                .rotateControl(true)
                .scaleControl(true)
                .streetViewControl(true)
                .zoomControl(true)
                .zoom(2);

        map = mapView.createMap(mapOptions);

        map.getMarkerClusterer().setGridSize(20);

        LatLongBounds bounds = new LatLongBounds();

        for (final ComponentManagerInfo cmi : cmiManager.getComponentManagerInfos()) {
            SfaAuthority auth = cmi.getAuthority();
            if (auth.getLocation() != null) {
                LatLong authLatLong =
                        new LatLong(auth.getLocation().getLatitude(), auth.getLocation().getLongitude());

                bounds.extend(authLatLong);

                MarkerOptions authMarkerOptions = new MarkerOptions();
                authMarkerOptions.position(authLatLong);
                authMarkerOptions.title(auth.getHrn());

                Marker authMarker = new Marker(authMarkerOptions);

                LOG.info("Adding marker {} at location {}", auth.getHrn(), auth.getLocation());

                map.addClusterableMarker(authMarker);

                map.addUIEventHandler(authMarker, UIEventType.click, obj -> selectTestbed(cmi));
            }
        }

       // map.fitBounds(bounds);
    }

    private void selectTestbed(ComponentManagerInfo cmi) {
        SfaAuthority auth = cmi.getAuthority();
        map.panTo(new LatLong(auth.getLocation().getLatitude(), auth.getLocation().getLongitude()));

        testbedNameLabel.setText(auth.getHrn());
        testbedCountryLabel.setText("Country: " + auth.getLocation().getCountry());

        selectedCmi.setValue(cmi);
    }

    public void setCmiManager(CMIManager CMIManager) {
        this.cmiManager = CMIManager;
        tryInitializeMap();
    }

    @FXML
    private void selectTestbed() {
        result = selectedCmi.get();
        ((Stage) selectTestbedButton.getScene().getWindow()).close();
    }

    public ComponentManagerInfo getResult() {
        return result;
    }
}
