package be.iminds.ilabt.jfed.experimenter_gui.gts_editor;

import be.iminds.ilabt.jfed.gts.GtsDslConstants;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.*;
import javafx.geometry.Point2D;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 5/21/2015
 * Time: 15:36
 */
public class GTSModelEditor {
    private static final Logger LOG = LoggerFactory.getLogger(GTSModelEditor.class);
    private final EditableGtsDsl editableGtsDsl;
    private final FXModelRspec modelRspec;

    private final FXRspecFactory rspecFactory = FXRspecFactory.getInstance();

    public GTSModelEditor(EditableGtsDsl editableGtsDsl) {
        this.editableGtsDsl = editableGtsDsl;
        this.modelRspec = editableGtsDsl.getModel();
    }

    public FXModelRspec getModelRspec() {
        return modelRspec;
    }

    public FXRspecNode addHost(Point2D location) {
        FXRspecNode node = rspecFactory.createNodeWithClientId(modelRspec, nextNodeName("host"));
        node.setComponentManagerId(GtsDslConstants.GTS_URN_HOST_CM);
        node.setComponentId(GtsDslConstants.GTS_URN_LOCATION_CPH);

        node.setEditorX(location.getX());
        node.setEditorY(location.getY());

        modelRspec.getNodes().add(node);

        return node;
    }


    public FXRspecNode addPort(Point2D location) {
        FXRspecNode node = rspecFactory.createNodeWithClientId(modelRspec, nextNodeName("port"));
        node.setComponentManagerId(GtsDslConstants.GTS_URN_PORT_CM);

        node.setEditorX(location.getX());
        node.setEditorY(location.getY());

        modelRspec.getNodes().add(node);

        return node;
    }


    private String nextNodeName(String prefix) {
        int c = 0;
        String res = prefix + c;
        while (modelRspec.getNodeByClientId(res) != null)
            res = prefix + (++c);

        return res;
    }

    public FXRspecLink addLink(FXRspecNode node1, FXRspecNode node2) {
        FXRspecLink rspecLink = rspecFactory.createLinkWithClientId(modelRspec, modelRspec.nextLinkName(), "lan");

        addNodeToLink(node1, rspecLink);
        addNodeToLink(node2, rspecLink);

        modelRspec.getLinks().add(rspecLink);

        return rspecLink;
    }

    /**
     * Creates a new FXRspecInterface to connect an FXRspecLink and an FXRspecNode.
     * Does additional configuration on this interface (or other interfaces on the link) if
     * required by the configuration-file.
     */
    private FXRspecInterface addNodeToLink(FXRspecNode node, FXRspecLink link) {
        FXRspecInterface iface = rspecFactory.createInterface(node,
                link,
                modelRspec.nextIfaceName(node));
        LOG.trace("Added node {} to link {} with iface {}", node.getUniqueId(), link.getUniqueId(), iface.getClientId());

        return iface;
    }
}
