package be.iminds.ilabt.jfed.experimenter_gui.slice;

/**
 * User: twalcari
 * Date: 1/30/2015
 * Time: 11:05
 */
public enum SliceState {

    /**
     * Initial status of a new Slice.
     * <p/>
     * The slice is to be created at the slice authority (It is currently unallocated).
     * <p/>
     * Next step in lifecycle will be @link{CREATED}
     */
    PENDING,

    /**
     * Initial status of an existing slice, of which we know nothing except for it's URN.
     *
     * While in this state, basic information about the slice is reconstructed by contacting the slice authority.
     * After that, all necessary sources are contacted to reconstruct the full state of all the slivers.
     *
     * When all sources have replied, the slice will transfer to the state @link{CREATED}
     */
    RESTORING,
    /**
     * The slice is allocated, but no provisioning has been done.
     *
     * If we are expected to do provisioning (e.g. experiment.getNewRequestRspecSource() != null) then:
     *    - if the resources are only available in the future: transfer to @link{FUTURE_RESERVATION}
     *    - otherwise, transfer to @link{PROVISIONING}
     *
     * If we have no provisioning to do, we can directly transfer to @link{TESTING} the existing resources
     */
    CREATED,

    /**
     * The resources must be reserved in the future.
     *
     * Once the reservation has been made. The state will transfer to @link{FUTURE_RESERVATION}, where it will wait
     * until the reservation start time is met. When this state finishes successfully, it will set
     * @link{Experiment.reservationMade}  to true.
     */
    RESERVING,
    /**
     * The resources of the slice are not yet provisionable, we have to wait until the correct time arrives.
     * <p/>
     * Once the reservation start time passes, the slice can transfer to PROVISIONING.
     * <p/>
     * The slice contains a manifest of the resources it has been promised.
     */
    FUTURE_RESERVATION,

    /**
     * The resources of the slice are currently being provisioned.
     * During this status, the slice receives the manifests of all slivers.
     * <p/>
     * When provisioning is done, the resources must be tested (transfer to status READY).
     */
    PROVISIONING,

    /**
     * All resources present should be tested if they are available.
     *
     * This state will only be set when restoring a slice.
     */
    TESTING,

    /**
     * The slice is fully provisioned and tested.
     * <p/>
     * The slice remains in this state, until it's expiration time. After that, it moves to the state EXPIRED_OR_EMPTY.
     */
    READY,

    /**
     * One or more slivers and/or slice is expiring. We need to check it to make sure it really happened!
     */
    EXPIRING,
    /**
     * The resources of the slice have expired, or there are no more active resources in the experiment
     */
    EXPIRED_OR_EMPTY,

    /**
     * If there occurs a fatal error, the slice transfers to FAILED. The user cannot use this slice anymore.
     */
    FAILED,

    /**
     * If there occurs an error when making the reservation (one or more resources could not be reserved),
     * the user should be notified and given the opportunity to delete the reservation.
     */
    RESERVATION_FAILED
}
