package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import com.google.common.base.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 10:30
 */
public class CheckExpirationTask extends ExperimentTask {
    private static final Logger LOG = LoggerFactory.getLogger(CheckExpirationTask.class);
    protected final HighLevelController.GetStatusUntilReadyOrFailTask getStatusUntilReadyOrFailTask;
    protected final SfaAuthority authority;

    CheckExpirationTask(Experiment experiment, SfaAuthority authority, HighLevelController hlc) {
        this(experiment, authority, 0, hlc);
    }

    CheckExpirationTask(Experiment experiment, SfaAuthority authority, int initialWaitMillis, HighLevelController hlc) {
        this(experiment, createAndSubmitTask(experiment, authority, hlc, initialWaitMillis));
    }

    CheckExpirationTask(Experiment experiment, HighLevelController.GetStatusUntilReadyOrFailTask getStatusUntilReadyOrFailTask) {
        super(experiment);
        this.getStatusUntilReadyOrFailTask = getStatusUntilReadyOrFailTask;
        this.authority = getStatusUntilReadyOrFailTask.getAuthority();

        updateTitle(String.format("Checking expiration of resources on %s", authority.getHrn()));
    }

    @Override
    protected TaskStatusIndicator.Status executeTask() {
        int punctuationCount = -1;
        while (!getStatusUntilReadyOrFailTask.isFinished()) {
            if (getStatusUntilReadyOrFailTask.getMsUntilNextCheck() == 0) {
                updateMessage(String.format("Checked status %d times. Checking%s", getStatusUntilReadyOrFailTask.getTriesCount(), Strings.repeat(".", 1 + ++punctuationCount % 3)));
            } else {
                updateMessage(String.format("Checked status %d times. Next check in %d seconds",
                        getStatusUntilReadyOrFailTask.getTriesCount(),
                        roundUp(getStatusUntilReadyOrFailTask.getMsUntilNextCheck(),
                                1000)));
                punctuationCount = -1;
            }

            try {
                Thread.sleep(1000);
            } catch (InterruptedException ignored) {
            }
        }


        //the GetStatusUntilReadyOrFailTask is finished, we can set the final status of this task

        updateMessage(getStatusUntilReadyOrFailTask.getStatus());

        Sliver sliver = experiment.getSlice().findSlivers(authority).get(0);

        LOG.debug("Stopped listening for results of status calls after receiving status {} ", getStatusUntilReadyOrFailTask.getStatusDetails().getGlobalStatus());

        onFinished();

        switch (sliver.getStatus().getGlobalStatus()) {
            case FAIL: {
                return TaskStatusIndicator.Status.FAILED;
            }
            case UNALLOCATED:
            case READY:
            default: {
                return TaskStatusIndicator.Status.SUCCESS;
            }
        }
    }

    protected void onFinished() {

    }

    @Override
    public String toString() {
        return "StatusUntilReadyTask{" +
                "authority=" + authority +
                '}';
    }

    private static HighLevelController.GetStatusUntilReadyOrFailTask createAndSubmitTask(Experiment experiment, SfaAuthority authority, HighLevelController hlc, int initialWaitMillis) {
        HighLevelController.GetStatusUntilReadyOrFailTask getStatusUntilReadyOrFailTask =
                hlc.getGetStatusUntilReadyOrFailTask(authority, experiment.getSlice());

        if (initialWaitMillis == 0)
            hlc.submitTask(getStatusUntilReadyOrFailTask);
        else {
            hlc.scheduleTask(getStatusUntilReadyOrFailTask, initialWaitMillis);
        }
        return getStatusUntilReadyOrFailTask;
    }


    private static long roundUp(long num, long divisor) {
        return (num + divisor - 1) / divisor;
    }

}
