package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.application.Platform;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.scene.control.*;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.FontAwesome;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 14:43
 */
public class CreateDiskImageTask extends SingleCallExperimentTask implements CustomDetailsView {

    private static final int INITIAL_WAIT_MILLIS = 10_000;
    private final SfaAuthority authority;
    private final GeniUrn sliverId;
    private final String imageName;

    private final ExperimentController experimentController;
    private final ExperimentTasksFactory experimentTasksFactory;

    private final HighLevelController.CreateImageTask createImageTask;
    private boolean taskFinishedAndSuccess = false;

    CreateDiskImageTask(ExperimentController experimentController, SfaAuthority authority, FXRspecNode node, String imageName,
                        boolean global, boolean updatePrepare,
                        HighLevelController hlc, ExperimentTasksFactory experimentTasksFactory) {
        super(experimentController.getExperiment(), hlc);
        this.experimentController = experimentController;
        this.authority = authority;
        this.experimentTasksFactory = experimentTasksFactory;
        this.sliverId = node.getSliverId();
        this.imageName = imageName;

        this.createImageTask = hlc.getCreateImageTask(experiment.getSlice(), sliverId, imageName, global, updatePrepare);

        updateTitle("Create disk image from node " + node.getClientId());
    }


    @Override
    public Task createTask() {
        return createImageTask;
    }


    @Override
    public void onTaskFinished(TaskExecution taskExecution) {
        if (taskExecution.getState() == TaskExecution.TaskState.SUCCESS) {
            taskFinishedAndSuccess = true;

            experimentController.submitExperimentTask(experimentTasksFactory.createStatusUntilReadyTask(experiment, authority, INITIAL_WAIT_MILLIS));

            updateMessage(createImageTask.getResultDiskImageUrn().toString());

            showInfoDialog();
        }
    }

    @Override
    public EventHandler<Event> getCustomDetailsView() {
        if (taskFinishedAndSuccess)
            return event -> showInfoDialog();
        else
            return null;
    }

    private void showInfoDialog() {
        Platform.runLater(() -> {
            VBox container = new VBox();
            container.setSpacing(5);

            Label label = new Label("Your disk image is currently being created. " +
                    "This process typically takes around 15 minutes. " +
                    "You will receive an e-mail when the disk image is ready.\n\n" +
                    "If you want to use the image on a different testbed then the current, use the following RSpec snippet:");

            TextArea textArea = new TextArea();
            textArea.setText(String.format("<sliver_type name=\"raw-pc\">\n" +
                    "\t<disk_image name=\"%s\"/>\n" +
                    "</sliver_type>", createImageTask.getResultDiskImageUrl()));
            textArea.setPrefRowCount(3);
            textArea.setPrefColumnCount(40);
            textArea.setEditable(false);
            textArea.setMinHeight(85);

            Label label2 =
                    new Label("If however, you want to use this image on the same testbed, it is better to use:");
            TextArea textArea2 = new TextArea();
            textArea2.setText(String.format("<sliver_type name=\"raw-pc\">\n" +
                    "\t<disk_image name=\"%s\"/>\n" +
                    "</sliver_type>", createImageTask.getResultDiskImageUrn()));
            textArea2.setPrefRowCount(3);
            textArea2.setPrefColumnCount(40);
            textArea2.setEditable(false);
            textArea2.setMinHeight(85);

            container.getChildren().addAll(label, textArea, label2, textArea2);

            DialogPane dialogPane = new DialogPane();
            dialogPane.setContent(container);
            dialogPane.setHeaderText("Your Disk Image is being prepared");
            dialogPane.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.HDD_ALT, Color.GRAY));
            dialogPane.getButtonTypes().setAll(ButtonType.OK);

            Dialog<ButtonType> dialog = new Dialog<>();
            dialog.setTitle("Your Disk Image is being prepared");
            dialog.setDialogPane(dialogPane);

            dialog.show();

        });
    }
}
