package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticAggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * User: twalcari
 * Date: 3/6/2015
 * Time: 16:12
 */
public class EditSshKeysTaskGenerator  extends ExperimentTaskGenerator{
    private static final Logger LOG = LoggerFactory.getLogger(EditSshKeysTaskGenerator.class);

    private static final int INITIAL_WAIT_MILLIS = 1_000;
    private final List<UserSpec> userSpecs;
    private final HighLevelController hlc;
    private final AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory;
    private final ExperimentTasksFactory experimentTasksFactory;


    public EditSshKeysTaskGenerator(ExperimentController experimentController, List<UserSpec> userSpecs,
                                    HighLevelController hlc, AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory,
                                    ExperimentTasksFactory experimentTasksFactory) {
        super(experimentController);
        this.userSpecs = userSpecs;
        this.hlc = hlc;
        this.automaticAggregateManagerWrapperFactory = automaticAggregateManagerWrapperFactory;
        this.experimentTasksFactory = experimentTasksFactory;
    }

    @Override
    protected TaskStatusIndicator.Status generateExperimentTasks() {

        for (SfaAuthority authority : experiment.getSlice().getAuthorities()) {
            if(automaticAggregateManagerWrapperFactory.create(authority).isEditSshKeysSupported()){

                EditSshKeysTask editSshKeysTask = new EditSshKeysTask(experimentController.getExperiment(), authority, userSpecs, hlc);

                editSshKeysTask.addTaskFinishedCallback(status -> {
                    if(status == TaskStatusIndicator.Status.SUCCESS){
                        experimentController.submitExperimentTask(experimentTasksFactory.createStatusUntilReadyTask(experiment,authority, INITIAL_WAIT_MILLIS));
                    }
                });

                experimentController.submitExperimentTask(editSshKeysTask);


            }   else{
                LOG.info("Skipping updating SSH keys info on {}, as it is not supported", authority.getHrn());
            }
        }


        return TaskStatusIndicator.Status.SUCCESS;
    }

    private static class EditSshKeysTask extends SingleCallExperimentTask {

        private final SfaAuthority authority;
        private final List<UserSpec> userspec;


        EditSshKeysTask(Experiment experiment, SfaAuthority authority, List<UserSpec> userspec, HighLevelController hlc) {
            super(experiment, hlc);
            this.authority = authority;
            this.userspec = userspec;

            updateTitle(String.format("Updating SSH keys on %s for %d user(s).", authority.getName(), userspec.size()));
        }

        @Override
        public Task createTask() {
            return hlc.getEditSshKeysTask(experiment.getSlice(), authority, userspec);
        }
    }

}
