package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * User: twalcari
 * Date: 2/5/2015
 * Time: 15:32
 */
public abstract class  ExperimentTaskGenerator implements Runnable {
    private static final Logger LOG = LoggerFactory.getLogger(ExperimentTaskGenerator.class);

    protected final ExperimentController experimentController;
    protected final Experiment experiment;

    private final List<ExperimentTaskFinishedCallback> finishedCallbacks = new ArrayList<>();

    public ExperimentTaskGenerator(ExperimentController experimentController){
        this.experimentController = experimentController;
        this.experiment = experimentController.getExperiment();
    }

    @Override
    public final void run() {
        try {

            TaskStatusIndicator.Status status = generateExperimentTasks();

            finishedCallbacks.forEach(callback -> callback.onExperimentTaskFinished(status));
        }catch(Exception | AssertionError throwable){
            LOG.error("Caught an exception while executing ExpermentTaskGenerator '{}'", getClass().getName(), throwable);
        }
    }

    protected abstract TaskStatusIndicator.Status generateExperimentTasks();

    public ExperimentController getExperimentController() {
        return experimentController;
    }

    public void addTaskFinishedCallback(ExperimentTaskFinishedCallback callback) {
        finishedCallbacks.add(callback);
    }
}
