package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.event.Event;
import javafx.event.EventHandler;

import javax.annotation.Nullable;
import java.net.URL;

/**
 * User: twalcari
 * Date: 2/4/2015
 * Time: 10:14
 */
public class OpenConsoleTask extends SingleCallExperimentTask implements CustomDetailsView {

    private final Sliver sliver;
    private final HighLevelController.OpenConsoleTask openConsoleTask;

    OpenConsoleTask(Experiment experiment, Sliver sliver, HighLevelController hlc) {
        super(experiment, hlc);
        this.sliver = sliver;

        this.openConsoleTask = hlc.getOpenConsoleTask(sliver);

        updateTitle("Open Console for sliver " + sliver.getUrn().toString());
    }

    @Override
    public Task createTask() {
        return openConsoleTask;
    }

    @Override
    public void onTaskFinished(TaskExecution taskExecution) {
        if (taskExecution.getState() == TaskExecution.TaskState.SUCCESS) {
            assert (openConsoleTask.getUrl() != null);

            updateMessage(openConsoleTask.getUrl().toExternalForm());
        }
    }

    public URL getConsoleURL() {
        return openConsoleTask.getUrl();
    }

    @Nullable
    @Override
    public EventHandler<Event> getCustomDetailsView() {
        if (isFinished()) {
            return event -> {
                if (openConsoleTask.getUrl() != null) {
                    JFDialogs.create()
                            .message("Console has been opened in browser. URL is: " + (
                                            openConsoleTask.getUrl() != null ?
                                                    openConsoleTask.getUrl().toExternalForm() : "null"
                                    )
                            ).allowCopyMessage()
                            .showWarning();
                }
            };
        } else
            return null;


    }
}
