package be.iminds.ilabt.jfed.experimenter_gui.bugreporting;

import be.iminds.ilabt.jfed.call_log_output.CallReportWriter;
import be.iminds.ilabt.jfed.call_log_output.LogOutput;
import be.iminds.ilabt.jfed.experimenter_gui.connectivity_tester.ConnectivityTester;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.highlevel.HighLevelModule;
import be.iminds.ilabt.jfed.highlevel.controller.JavaFXTaskThread;
import be.iminds.ilabt.jfed.highlevel.history.StitchingReportHistory;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.stitcher.StitchingReport;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsCache;
import be.iminds.ilabt.jfed.logging.JFedLoggerManager;
import be.iminds.ilabt.jfed.lowlevel.userloginmodel.UserLoginModelManager;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.collections.FXCollections;
import javafx.collections.ObservableMap;
import org.testng.annotations.Guice;
import org.testng.annotations.Test;

import javax.inject.Inject;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.StringReader;
import java.util.Collections;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;


@Test
@Guice(modules = HighLevelModule.class)
public class BugReportComposerTest {

    @Inject
    private BugReportReaderFactory bugReportReaderFactory;

    @Inject
    private CallReportWriter callReportWriter;

    @Inject
    private ApiCallDetailsCache cache;

    @Test
    public void testStitchingReports() throws Exception {
        //this tests adding StitchingReports to the bugreport. Nothing else about the BugReport is reliably tested.
        StitchingReport fakeStitchingReport = new StitchingReport("First",cache);
        fakeStitchingReport.changeState("First").debug("Testing 1");
        fakeStitchingReport.changeState("Second").warn("OK 2");
        fakeStitchingReport.changeState("Third").error("Still OK 3");
        StitchingReportHistory stitchingReportHistory = new StitchingReportHistory();
        stitchingReportHistory.addStitchingReport(fakeStitchingReport);

        JavaFXTaskThread javaFXTaskThread = mock(JavaFXTaskThread.class);
        UserLoginModelManager userLoginModelManager = mock(UserLoginModelManager.class);
        EasyModel easyModel = mock(EasyModel.class);
        ConnectivityTester connectivityTester = mock(ConnectivityTester.class);
        ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers = FXCollections.observableHashMap();
        JFedGuiPreferences jFedPreferences = mock(JFedGuiPreferences.class);
        JFedLoggerManager jFedLoggerManager = mock(JFedLoggerManager.class);

        when(userLoginModelManager.getUserUrnString()).thenReturn("testuserurn");
        when(userLoginModelManager.getUserLoginModelType()).thenReturn(UserLoginModelManager.UserLoginModelType.KEY_CERT_EXTERNAL_INFO); //prevent requesting certificate

        when(connectivityTester.getTestsFinished()).thenReturn(true);

        when(jFedLoggerManager.getBugReportMemoryAppender()).thenReturn(null);
        when(javaFXTaskThread.getCallsCount()).thenReturn(0);
        when(connectivityTester.getTestResults()).thenReturn(Collections.emptyMap());
//        when(experimentViewControllers.size()).thenReturn(0);
        when(jFedPreferences.getPropertiesFile()).thenReturn(File.createTempFile("empty", ".properties"));
        when(javaFXTaskThread.getAllTasks()).thenReturn(FXCollections.emptyObservableList());
        when(connectivityTester.getTestResults()).thenReturn(Collections.emptyMap());
        when(easyModel.getSlices()).thenReturn(FXCollections.emptyObservableList());

        BugReportComposer bugReportComposer = new BugReportComposer(
                javaFXTaskThread,
                userLoginModelManager,
                easyModel,
                connectivityTester,
                experimentViewControllers,
                jFedPreferences,
                jFedLoggerManager,
                callReportWriter, stitchingReportHistory);
        bugReportComposer.setDescription("Test Description");
        bugReportComposer.setEmail("Test Email");

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        bugReportComposer.writeToOutputStream(baos);
        baos.close();
        String xml = new String(baos.toByteArray(), "UTF-8");
        System.out.println("Created BugReport xml: " + xml);

        //just a quick check to see if XML looks OK
        assertThat(xml, containsString("stitchingreports"));
        assertThat(xml, containsString("Still OK 3"));

        //now reconstruct
        BugReportReader reader =  bugReportReaderFactory.createBugReportReader(new StringReader(xml));

        BugReport bugReport = reader.call();

        assertThat(bugReport, is(notNullValue()));
        assertThat(bugReport.getStitchingReports(), hasSize(1));
        StitchingReport reconstructedReport = bugReport.getStitchingReports().get(0);
        assertThat(reconstructedReport, is(notNullValue()));
        assertThat(reconstructedReport.getStates(), hasSize(3));
        assertThat(reconstructedReport.getLastState("First").getCalls(), hasSize(0));
        assertThat(reconstructedReport.getLastState("First").getLogLines(), hasSize(1));
        assertThat(reconstructedReport.getLastState("First").getLogLines().get(0).getType(), equalTo(LogOutput.LogLineType.DEBUG));
        assertThat(reconstructedReport.getLastState("First").getLogLines().get(0).getText(), equalTo("Testing 1"));
        assertThat(reconstructedReport.getLastState("Second").getLogLines(), hasSize(1));
        assertThat(reconstructedReport.getLastState("Second").getLogLines().get(0).getText(), equalTo("OK 2"));
        assertThat(reconstructedReport.getLastState("Second").getLogLines().get(0).getType(), equalTo(LogOutput.LogLineType.WARN));
        assertThat(reconstructedReport.getLastState("Third").getLogLines(), hasSize(1));
        assertThat(reconstructedReport.getLastState("Third").getLogLines().get(0).getText(), equalTo("Still OK 3"));
        assertThat(reconstructedReport.getLastState("Third").getLogLines().get(0).getType(), equalTo(LogOutput.LogLineType.ERROR));
    }
}
