package be.iminds.ilabt.jfed.highlevel.controller;

import be.iminds.ilabt.jfed.CorePreferencesModule;
import be.iminds.ilabt.jfed.MockHttpRepliesTestCoreModule;
import be.iminds.ilabt.jfed.TestAuthoritiesModule;
import be.iminds.ilabt.jfed.TestUserModule;
import be.iminds.ilabt.jfed.highlevel.HighLevelModule;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.TestCertHelper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.lowlevel.connection.MockReplyLibrary;
import com.google.inject.Guice;
import com.google.inject.Injector;
import javafx.embed.swing.JFXPanel;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;

/**
 * HighLevelControllerTest
 */
public class HighLevelControllerMockHttpRepliesTest {
    private SfaAuthority testAuth;

    private MockReplyLibrary mockReplyLibrary;

    private AuthorityListModel authorityListModel;
    private AuthorityList authorityList;
    private JFedConnectionProvider connectionProvider;
    private GeniUserProvider geniUserProvider;
    private EasyModel easyModel;
    private Logger logger;
    private TaskThread tt;
    private HighLevelController highLevelController;


    private Injector injector;

    @BeforeMethod
    public void setUp() throws Exception {
        //initialize JavaFX
        new JFXPanel();

        mockReplyLibrary = new MockReplyLibrary();

        MockHttpRepliesTestCoreModule testCoreModule =  new MockHttpRepliesTestCoreModule();
        testCoreModule.addMockReply(mockReplyLibrary.getVersion3());

        try {
            injector = Guice.createInjector(testCoreModule,
                    new TestUserModule(),
                    new CorePreferencesModule(),
                    new TestAuthoritiesModule(),
                    new HighLevelModule());
        } catch (Exception | AssertionError e) {
            throw new RuntimeException("Error setting up injector", e);
        }

        TestCertHelper testCertHelper = injector.getInstance(TestCertHelper.class);
        this.authorityListModel = injector.getInstance(AuthorityListModel.class);
        this.authorityList = injector.getInstance(AuthorityList.class);
        this.tt = injector.getInstance(TaskThread.class);
        this.connectionProvider = injector.getInstance(JFedConnectionProvider.class);
        this.geniUserProvider = injector.getInstance(GeniUserProvider.class);
        this.logger = injector.getInstance(Logger.class);
        this.easyModel = injector.getInstance(EasyModel.class);
        this.highLevelController = injector.getInstance(HighLevelController.class);

        testAuth = authorityListModel.getByUrnExact(TestCertHelper.am1Urn);
    }

    @Test
    public void basicHighLevelControllerTest() throws InterruptedException {
        CountDownLatch countdownLatch = new CountDownLatch(2);

        Task t = highLevelController.getVersion(testAuth);
        TaskExecutionFinishedCallback callback = new TaskExecutionFinishedCallback() {
            @Override
            public void onTaskExecutionFinished(Task task, TaskExecution taskExecution, TaskExecution.TaskState state) {
                System.out.println("GetVersion completed");
                countdownLatch.countDown();
            }
        };
        tt.addTask(t, callback);

        countdownLatch.countDown();
        countdownLatch.await(1, TimeUnit.SECONDS);

        assertThat("getVersion was not called exactly once", mockReplyLibrary.getVersion3().usageCount(), equalTo(1));
    }
}
