package be.iminds.ilabt.jfed.highlevel.stitcher;

import be.iminds.ilabt.jfed.CorePreferencesModule;
import be.iminds.ilabt.jfed.MockHttpRepliesTestCoreModule;
import be.iminds.ilabt.jfed.TestAuthoritiesModule;
import be.iminds.ilabt.jfed.TestUserModule;
import be.iminds.ilabt.jfed.call_log_output.SerializableApiCallDetails;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.model.UserSpecHelper;
import be.iminds.ilabt.jfed.log.ApiCallDetails;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.log.ResultListener;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsCache;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsRef;
import be.iminds.ilabt.jfed.log.cache.MemoryApiCallDetailsCache;
import be.iminds.ilabt.jfed.lowlevel.AnyCredential;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.RetrySettings;
import be.iminds.ilabt.jfed.lowlevel.TestCertHelper;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticAggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.lowlevel.connection.MockReplyLibrary;
import be.iminds.ilabt.jfed.lowlevel.connection.MockSfaConnection;
import be.iminds.ilabt.jfed.lowlevel.stitching.VlanRange;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.JavaFXLogger;
import com.google.inject.Guice;
import com.google.inject.Injector;
import org.slf4j.LoggerFactory;

import javax.inject.Singleton;
import java.io.File;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertNotNull;

/**
 * StitchingTestSetup
 *
 * A collection of fake stitching servers and configuration for it, that is used for testing stitching.
 */
public class StitchingTestSetup {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(StitchingTestSetup.class);

    private final StitchingTestSetupConfig conf;

    public SfaAuthority testAm1Auth;
    public SfaAuthority testAm2Auth;
    public SfaAuthority testScsAuth;

    private JFedPreferences jFedPreferences;
    private AuthorityListModel authorityListModel;
    private AuthorityList authorityList;
    private JFedConnectionProvider connectionProvider;
    private GeniUserProvider geniUserProvider;
    private EasyModel easyModel;
    private Logger logger;
    private TaskThread tt;
    private HighLevelController highLevelController;
    private StitchingReportFactory stitchingReportFactory;

    private AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory;
    //    private AutomaticAggregateManagerWrapper automaticAggregateManagerWrapperMock;

    private Injector injector;

    private Slice slice;

    private TestStitchingXmlRpcHelper testStitchingXmlRpcHelper;

    private List<MockSfaConnection.MockXmlRpcRequestInfo> noMockXmlRpcRequestInfo;
    private List<MockSfaConnection.MockXmlRpcHistoryItem> mockXmlRpcHistory;

    private TestStitchingAMServer mockAm1;
    private TestStitchingAMServer mockAm2;
    private TestStitchingAMServer mockAm3;

    public String advertisedAvailableAm1;
    public String advertisedAvailableAm2;
    public String advertisedAvailableAm3;
    public String realAvailableAm1;
    public String realAvailableAm2;
    public String realAvailableAm3;

    private ApiCallDetailsCache cache;

    public StitchingTestSetup(StitchingTestSetupConfig conf) {
        this.conf = conf;

        MockHttpRepliesTestCoreModule testCoreModule =  new MockHttpRepliesTestCoreModule();

        advertisedAvailableAm1 = "197,300-305,750-755";
        advertisedAvailableAm2 = "197,300-305,750-755";
        advertisedAvailableAm3 = "197,300-305,750-755";
        realAvailableAm1 = "300-302,305,750-755";
        realAvailableAm2 = "197,304-305,750-755";
        realAvailableAm3 = "197,304-305,750-755";

        if (conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.am1NoMatchingVlansAm2)) {
            realAvailableAm1 = "300-302,753-755";     //count=6
            realAvailableAm2 = "197,304-305,751-752"; //count=5
        }

        MockReplyLibrary mockReplyLibrary = new MockReplyLibrary();
        testCoreModule.addMockReply(mockReplyLibrary.getVersion3());
        testCoreModule.addMockReply(new MockSfaConnection.AbstractMockXmlRpcReply() {
            @Override
            public MockSfaConnection.MockXmlRpcAnswer matches(MockSfaConnection.MockXmlRpcRequestInfo mockXmlRpcRequestInfo) {
                if (!mockXmlRpcRequestInfo.serverUrl.equals(TestAuthoritiesModule.urlC_scs))
                    return null;
                if (mockXmlRpcRequestInfo.method.equals("ComputePath") && mockXmlRpcRequestInfo.params.size() > 0) {
                    if (conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.scsFailure))
                        return new MockSfaConnection.MockXmlRpcAnswer(testStitchingXmlRpcHelper.wrapErrorReply(5, "SCS failure"));

                    if (conf.amCount() < 3 || conf.am(3).noStitching) {
                        if (conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.am2DependsOnAm1))
                            return new MockSfaConnection.MockXmlRpcAnswer(testStitchingXmlRpcHelper.create2DepAmScsHttpReply(
                                    advertisedAvailableAm1, advertisedAvailableAm2,
                                    "304", "304", conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.hasFakeAm3), conf.amCount() > 2 ? conf.am(3).noStitching : false));
                        else
                            return new MockSfaConnection.MockXmlRpcAnswer(testStitchingXmlRpcHelper.create2IndepAmScsHttpReply(
                                    advertisedAvailableAm1, advertisedAvailableAm2,
                                    "304", "304", conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.hasFakeAm3), conf.amCount() > 2 ? conf.am(3).noStitching : false));
                    } else {
                        //3 or more am's
                        assert conf.amCount() == 3;
                        assert conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.am2DependsOnAm1);
                        return new MockSfaConnection.MockXmlRpcAnswer(testStitchingXmlRpcHelper.create3DepAmScsHttpReply(
                                advertisedAvailableAm1, advertisedAvailableAm2, advertisedAvailableAm3,
                                "304", "304", "304"));
                    }
                }
                return null;
            }
        });

        if (conf.am(1).hasV2) {
            mockAm1 = new TestStitchingAMv2Server(TestAuthoritiesModule.urlA_am2, TestCertHelper.am1Urn, TestCertHelper.authority1Hostname,
                    "stitchIface1", new VlanRange(realAvailableAm1), conf.am(1), null);
            testCoreModule.addMockReply(mockAm1);
        }
        if (conf.am(1).hasV3) {
            mockAm1 = new TestStitchingAMv3Server(TestAuthoritiesModule.urlA_am3, TestCertHelper.am1Urn, TestCertHelper.authority1Hostname,
                    "stitchIface1", new VlanRange(realAvailableAm1), conf.am(1), null);
            testCoreModule.addMockReply(mockAm1);
        }

        if (conf.am(2).hasV2) {
            mockAm2 = new TestStitchingAMv2Server(TestAuthoritiesModule.urlB_am2, TestCertHelper.am2Urn, TestCertHelper.authority2Hostname,
                    "stitchIface2", new VlanRange(realAvailableAm2), conf.am(2), conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.am2DependsOnAm1) ? mockAm1 : null);
            testCoreModule.addMockReply(mockAm2);
        }
        if (conf.am(2).hasV3) {
            mockAm2 = new TestStitchingAMv3Server(TestAuthoritiesModule.urlB_am3, TestCertHelper.am2Urn, TestCertHelper.authority2Hostname,
                    "stitchIface2", new VlanRange(realAvailableAm2), conf.am(2), conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.am2DependsOnAm1) ? mockAm1 : null);
            testCoreModule.addMockReply(mockAm2);
        }

        if (conf.amCount() >= 3) {
            if (conf.am(3).hasV2) {
                mockAm3 = new TestStitchingAMv2Server(TestAuthoritiesModule.urlC_am2, TestCertHelper.am3Urn, TestCertHelper.authority3Hostname,
                        "stitchIface3", new VlanRange(realAvailableAm3), conf.am(3), null/*dependency*/);
                testCoreModule.addMockReply(mockAm3);
            }
            if (conf.am(3).hasV3) {
                mockAm3 = new TestStitchingAMv3Server(TestAuthoritiesModule.urlC_am3, TestCertHelper.am3Urn, TestCertHelper.authority3Hostname,
                        "stitchIface3", new VlanRange(realAvailableAm3), conf.am(3), null/*dependency*/);
                testCoreModule.addMockReply(mockAm3);
            }
        }

        testCoreModule.addOnNoMockReply(new MockSfaConnection.OnMockXmlRpcListener() {
            @Override
            public void onNoMockFor(MockSfaConnection.MockXmlRpcRequestInfo mockXmlRpcRequestInfo) {
                noMockXmlRpcRequestInfo.add(mockXmlRpcRequestInfo);
            }
        });
        testCoreModule.addMockXmlRpcHistoryListener(new MockSfaConnection.MockXmlRpcHistoryListener() {
            @Override
            public void onMockXmlRpcHistory(MockSfaConnection.MockXmlRpcHistoryItem item) {
                synchronized (mockXmlRpcHistory) {
                    mockXmlRpcHistory.add(item);
                }
            }
        });

        try {
            injector = Guice.createInjector(testCoreModule,
                    new TestUserModule(),
                    new CorePreferencesModule(),
                    new TestAuthoritiesModule(
                            new TestAuthoritiesModule.AMConfig(conf.am(1).hasV2, conf.am(1).hasV3, conf.am(1).advUpToDate, conf.am(1).supportsAny),
                            new TestAuthoritiesModule.AMConfig(conf.am(2).hasV2, conf.am(2).hasV3, conf.am(2).advUpToDate, /*conf.am(2).supportsAny*/true), //usefull for testing that sometimes any is not reported as unsupported
                            conf.amCount() < 3 ? new TestAuthoritiesModule.AMConfig()/*defaults*/ :
                                    new TestAuthoritiesModule.AMConfig(conf.am(3).hasV2, conf.am(3).hasV3, conf.am(3).advUpToDate, conf.am(3).supportsAny)),
                    binder -> {
                        binder.bind(ApiCallDetailsCache.class).to(MemoryApiCallDetailsCache.class).in(Singleton.class);
                        binder.bind(Logger.class).to(JavaFXLogger.class).in(Singleton.class);
                    });
        } catch (Exception | AssertionError e) {
            throw new RuntimeException("Error setting up injector", e);
        }

        TestCertHelper testCertHelper = injector.getInstance(TestCertHelper.class);
        testStitchingXmlRpcHelper = new TestStitchingXmlRpcHelper(testCertHelper);


        TestStitchingSAServer mockSa = new TestStitchingSAServer(TestAuthoritiesModule.urlA_pgSa1, TestCertHelper.saUrn, TestCertHelper.authority1Hostname, testCertHelper);
        testCoreModule.addMockReply(mockSa);


        automaticAggregateManagerWrapperFactory =
                injector.getInstance(AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory.class);

        //when mocking aggregateManagerWrapper, this could be used:
//        SfaAuthority am1Auth = mock(SfaAuthority.class);
//        when(am1Auth.getUrnString()).thenReturn(TestCertHelper.am1Urn);
//        SfaAuthority am2Auth = mock(SfaAuthority.class);
//        when(am2Auth.getUrnString()).thenReturn(TestCertHelper.am2Urn);
//
//        AutomaticAggregateManagerWrapper am1Mock = automaticAggregateManagerWrapperFactory.create(am1Auth);
//        AutomaticAggregateManagerWrapper am2Mock = automaticAggregateManagerWrapperFactory.create(am2Auth);
//        assert am1Mock != null;
//        when(am1Mock.getVersion()).thenReturn(new Hashtable());
//        when(am1Mock.hasSeperateAllocateAndProvision()).thenReturn(true);
//        when(am1Mock.allocate(any(GeniUrn.class), any(List.class), any(String.class)/*request rspec*/, any(Date.class))).
//                thenReturn(testStitchingRspecHelper.createAm1Manifest());
//
//        assert am2Mock != null;
//        when(am2Mock.getVersion()).thenReturn(new Hashtable());
//        when(am2Mock.hasSeperateAllocateAndProvision()).thenReturn(false);
//        when(am2Mock.createSliver(any(GeniUrn.class), any(List.class), any(String.class)/*request rspec*/, any(List.class)/*userUrns*/, any(Date.class))).
//                thenReturn(testStitchingRspecHelper.createAm2Manifest());

        this.authorityListModel = injector.getInstance(AuthorityListModel.class);
        this.authorityList = injector.getInstance(AuthorityList.class);
        this.tt = injector.getInstance(TaskThread.class);
        this.connectionProvider = injector.getInstance(JFedConnectionProvider.class);
        this.geniUserProvider = injector.getInstance(GeniUserProvider.class);
        this.logger = injector.getInstance(Logger.class);
        this.easyModel = injector.getInstance(EasyModel.class);
        this.highLevelController = injector.getInstance(HighLevelController.class);
        this.jFedPreferences = injector.getInstance(JFedPreferences.class);
        this.jFedPreferences.setRetrySettings(new RetrySettings(
                        1/*timeoutMaxRetries*/,  //only 1 retry on timeout allowed in testing.
                        10/*timeoutMsBeforeRetry*/,
                        100/* busyMaxRetries*/,
                        10/*busyMsBeforeRetry*/,
                        1/*possibleTemporaryErrorMaxRetries*/,
                        10/*possibleTemporaryErrorMsBeforeRetry*/)
        );
        this.stitchingReportFactory = injector.getInstance(StitchingReportFactory.class);
        this.cache = injector.getInstance(ApiCallDetailsCache.class);

        System.out.println("Retry settings in test mock setup: "+jFedPreferences.getRetrySettings());

        testAm1Auth = authorityListModel.getByUrnExact(TestCertHelper.am1Urn);
        testAm2Auth = authorityListModel.getByUrnExact(TestCertHelper.am2Urn);
        testScsAuth = authorityListModel.getByUrnExact(TestCertHelper.scsUrn);

        mockAm1.setAuthority(testAm1Auth);
        mockAm2.setAuthority(testAm2Auth);

        Instant expirationDate = Instant.now().plusSeconds(2*60*60);
        slice = mock(Slice.class);
        String requestRspecString = testStitchingXmlRpcHelper.createStitchRequest();
        RequestRspecSource requestRspecSource = new RequestRspecSource(requestRspecString, ModelRspecType.BASIC);
        when(slice.hasCredential()).thenReturn(true);
        GeniUrn sliceUrn = GeniUrn.createGeniUrnFromEncodedParts(TestCertHelper.authority1Hostname, "slice", "test");
        when(slice.getUrn()).thenReturn(sliceUrn);
        when(slice.getUrnString()).thenReturn(sliceUrn+"");
        when(slice.getRequestRspec()).thenReturn(requestRspecSource);
        when(slice.getExpirationDate()).thenReturn(expirationDate);

        assert automaticAggregateManagerWrapperFactory != null;
    }

    private boolean gaveUpOnStitching = false;
    private boolean timeOut = false;
    private ParallelStitcher parallelStitcher;

    public void testStitching() throws InterruptedException {
        mockXmlRpcHistory = new ArrayList<>();
        noMockXmlRpcRequestInfo = new ArrayList<>();

        JavaFXLogger logger = easyModel.getLogger();

        //      //print all calls
        //        logger.addResultListener(new ResultListener() {
        //            @Override
        //            public void onResult(ApiCallDetails result) {
        //                System.out.println("logger saw: "+result);
        //            }
        //        });

        final List<ApiCallDetails> calls = Collections.synchronizedList(new ArrayList<>());
        logger.addResultListener(new ResultListener() {
            @Override
            public void onResult(ApiCallDetails result) {
                calls.add(result);
            }
        });

        assert slice.hasCredential() :
                "Must use a slice with a known credential: " + slice + "   all slices:" + easyModel.getSlices();

        List<UserSpec> userSpecs =
                UserSpecHelper.getMergedUserSpecs(geniUserProvider, easyModel, slice.getRequestRspec().getModelRspec());

        ResultListener resultListener = new ResultListener() {
            @Override
            public void onResult(ApiCallDetails result) {
                //todo
            }
        };

        assert slice.getExpirationDate() != null;

        parallelStitcher =
                new ParallelStitcher(tt, easyModel, geniUserProvider,
                        slice.getRequestRspec(), connectionProvider,
                        testScsAuth, highLevelController, userSpecs, slice.getExpirationDate(), authorityList,
                        jFedPreferences, automaticAggregateManagerWrapperFactory,
                        5l/*ms between status checks*/,
                        750l /*ms before giving up on status*/, //this was 100ms, but due to scheduling that sometimes was too short
                        10l/*ms before retry createsliver after delete*/,
                        JavaFXLogger.wrap(logger, resultListener), stitchingReportFactory);


        ParallelStitcher.GiveUpStitchingCallback giveUpStitchingCallback = new ParallelStitcher.GiveUpStitchingCallback() {
            @Override
            public boolean onGiveUpStitching(String reason) {
                gaveUpOnStitching = true;
                return true /*mustDeleteAll*/;
            }
        };

        //fake user credentials
        AnyCredential userCredential = mock(AnyCredential.class);
        when(userCredential.getCredentialXml()).thenReturn("fake credential for testing");

        Hashtable<String, Object> userCredentialV3Ht = new Hashtable<String, Object>();
        userCredentialV3Ht.put("geni_type", "geni");
        userCredentialV3Ht.put("geni_version", "3");
        userCredentialV3Ht.put("geni_value", "fake credential for testing");
        when(userCredential.getGeniV3Hashtable()).thenReturn(userCredentialV3Ht);
        List<AnyCredential> userCredentials = new ArrayList<>();
        userCredentials.add(userCredential);

        parallelStitcher.setGiveUpStitchingCallback(giveUpStitchingCallback);
        parallelStitcher.setUserCredentialAndSlice(userCredentials, slice);
        parallelStitcher.start();

        System.out.flush();
        System.err.flush();

        System.out.println("Parallel stitcher started with slice "+slice.getUrn());
        System.out.println("\n\n\n");

        timeOut = !parallelStitcher.waitUntilDone(5, TimeUnit.SECONDS); //since it's all fake, this should be very quick. But we still allow way too much time anyway.

        //wait a while for calls to finish, buffers to flush, and all threads to finish.
        Thread.yield();
        Thread.sleep(50);
        Thread.yield();
        Thread.sleep(50);
        Thread.yield();
        Thread.sleep(50);
        Thread.yield();
        Thread.sleep(50);
        Thread.yield();
        Thread.sleep(50);
        Thread.yield();

        System.out.flush();
        System.err.flush();
        System.out.println("\n\n\n");

        System.out.println("Call history:");
        int i = 0;
        for (MockSfaConnection.MockXmlRpcHistoryItem histItem : new ArrayList<>(mockXmlRpcHistory)) {
            System.out.println("Call " + i + ": " + histItem.requestInfo.method + " @ " + histItem.requestInfo.serverUrl);

            i++;
        }
        System.out.println("");
        for (MockSfaConnection.MockXmlRpcRequestInfo requestInfo : new ArrayList<>(noMockXmlRpcRequestInfo)) {
            System.out.println("No Mock reply for: " + requestInfo.method + " @ " + requestInfo.serverUrl);
        }

        System.out.println("\n\n\n");
        System.out.println("Test finished in with parallelStitcher.getGlobalPhase()=" + parallelStitcher.getGlobalPhase());

        System.out.flush();
        System.err.flush();
    }

    public void outputToFile() {
        String basename = "stitching-";
        if (conf.am(1).hasV2) basename += "2";
        if (conf.am(1).hasV3) basename += "3";
        basename += "-";
        if (conf.am(2).hasV2) basename += "2";
        if (conf.am(2).hasV3) basename += "3";
        if (conf.amCount() >= 3) {
            basename += "-";
            if (conf.am(3).hasV2) basename += "2";
            if (conf.am(3).hasV3) basename += "3";
        }
        basename += "-";
        basename += conf.am(1).noStitching ? "nostitch" : (conf.am(1).supportsAny ? "any" : "noany");
        basename += "-";
        basename += conf.am(2).noStitching ? "nostitch" : (conf.am(2).supportsAny ? "any" : "noany");
        if (conf.amCount() >= 3) {
            basename += "-";
            basename += conf.am(3).noStitching ? "nostitch" : (conf.am(3).supportsAny ? "any" : "noany");
        }

        for (StitchingTestSetupConfig.SpecialCase specialCase : conf.specialCases)
            switch (specialCase) {
                case am2DependsOnAm1: {
                    basename += "-2deps1";
                    break;
                }
                case am1NoMatchingVlansAm2: {
                    basename += "-am1noSameVlanAm2";
                    break;
                }
                case hasFakeAm3: {
                    basename += "-fake";
                    break;
                }
                default: {
                    basename += "-"+specialCase;
                    break;
                }
            }

        for (StitchingTestSetupConfig.SpecialReply specialReply : conf.am(1).specialReplies)
            basename += "-1s"+specialReply.ordinal();
        for (StitchingTestSetupConfig.SpecialReply specialReply : conf.am(2).specialReplies)
            basename += "-2s"+specialReply.ordinal();
        if (conf.amCount() >= 3)
            for (StitchingTestSetupConfig.SpecialReply specialReply : conf.am(3).specialReplies)
                basename += "-3s"+specialReply.ordinal();

        this.outputToFile(basename);
    }
    public void outputToFile(String basename) {
        File file = new File(basename+".html");
        parallelStitcher.getStitchingReport().toHtmlFile(file, true);
        System.out.println("Wrote " + basename + ".html");
        System.out.println("      " + file.getAbsolutePath());
        System.out.println("      " + file.toURI());
        System.out.flush();
        System.err.flush();

    }

    public void assertAllTestBasics() {
        for (MockSfaConnection.MockXmlRpcHistoryItem histItem : mockXmlRpcHistory) {
            if (!histItem.reply.isException())
                assertThat("A request was seen by the fake server as a test failure", histItem.reply.getString(), not(containsString("TEST BUG DETECTED")));
        }

        //basic checks, even for failed stitchings, these should succeed
        assertThat("Timed out waiting for Parallel stitcher to finish", !timeOut);
        assertThat("Encountered a request for which no answer was mocked: ", noMockXmlRpcRequestInfo, empty());
    }

    public void assertMostTestBasics() {
        assertAllTestBasics();

        List<ApiCallDetailsRef> allocateStateCallsRef = getStitchingReport().getAllStateCalls("ALLOCATE");

        List<SerializableApiCallDetails> allocateStateCalls = allocateStateCallsRef.stream().map(cache::getApiCallDetails).collect(Collectors.toList());

        List<SerializableApiCallDetails> deleteCalls = allocateStateCalls.stream().
                filter(c -> c.getGeniMethodName().startsWith("Delete")).collect(Collectors.toList());
        List<SerializableApiCallDetails> allocateCalls = allocateStateCalls.stream().filter(
                c -> c.getGeniMethodName().equals("Allocate") || c.getGeniMethodName().equals("CreateSliver")).collect(Collectors.toList());
        List<SerializableApiCallDetails> allocateStateStatusCalls = allocateStateCalls.stream().filter(
                c -> c.getGeniMethodName().contains("Status")).collect(Collectors.toList());
        assertThat("there should be at least one Allocate in the Allocate state",
                allocateCalls, hasSize(greaterThan(0)));
        assertThat("there should be at least one Allocate for every Delete",
                allocateCalls, hasSize(greaterThanOrEqualTo(deleteCalls.size())));

        boolean hasAnyAmv2 = conf.am(1).hasV2 || conf.am(2).hasV2;
        if (conf.amCount() >= 3) hasAnyAmv2 |= conf.am(3).hasV2;
        if (!hasAnyAmv2)
            assertThat("there should be no Status calls in the Allocate state if there are no AMv2 AMs",
                    allocateStateStatusCalls, hasSize(is(0)));

        //am1 does not want expires, am2 and scs do
        for (SerializableApiCallDetails allocateDetail : allocateCalls) {
            SfaAuthority auth = authorityListModel.getByUrnExact(allocateDetail.getAuthorityUrn());
            boolean addExpires = auth.hasSpecialCase(SfaAuthority.SpecialCase.ADD_EXPIRES_ATTRIBUTE_TO_REQUEST);

            //check if the ADD_EXPIRES_ATTRIBUTE_TO_REQUEST is correctly registered in the SfaAuthority
            if (allocateDetail.getAuthorityUrn().equals(TestCertHelper.am1Urn))
                assertThat(addExpires, is(true));
            if (allocateDetail.getAuthorityUrn().equals(TestCertHelper.am2Urn))
                assertThat(addExpires, is(false));
            if (allocateDetail.getAuthorityUrn().equals(TestCertHelper.scsUrn))
                assertThat(addExpires, is(true));

            String requestRspec = allocateDetail.getXmlRpcRequestJsonString(); //don't care that this is more than the RSpec, it's good enough for this test.
            boolean hasExpires = requestRspec.contains("expires=");
//            LOG.debug("Testing if "+auth.getNameForUrn()+" has expires. request RSpec = "+requestRspec);
            if (addExpires)
                assertThat(allocateDetail.getAuthorityUrn()+" should have expires in request RSpec", hasExpires, is(true));
            else
                assertThat(allocateDetail.getAuthorityUrn()+" should not have expires in request RSpec", hasExpires, is(false));
        }
    }

    public void assertSuccess(boolean allowDelete) {
        assertMostTestBasics();
        assertThat("Gave up on stitching. globalPhase is " + parallelStitcher.getGlobalPhase(), gaveUpOnStitching, is(false));
        assertThat(parallelStitcher.getGlobalPhase(), equalTo(ParallelStitcher.GlobalPhase.SUCCESS));

        if (conf.hasSpecialCase(StitchingTestSetupConfig.SpecialCase.am2DependsOnAm1)) {
            assertNotNull(mockAm1.getActiveVlan());
            assertNotNull(mockAm2.getActiveVlan());
            assertThat(mockAm2.getActiveVlan(), equalTo(mockAm1.getActiveVlan()));
        }

        if (!allowDelete)
            for (MockSfaConnection.MockXmlRpcHistoryItem histItem : mockXmlRpcHistory) {
                assertThat(histItem.requestInfo.method, not(containsString("Delete")));
            }

        System.out.flush();
        System.err.flush();
    }

    public void assertFailure() {
        assertMostTestBasics();
        assertThat("Did not give up on stitching. globalPhase is " + parallelStitcher.getGlobalPhase(), gaveUpOnStitching, is(true));
        assertThat(parallelStitcher.getGlobalPhase(), equalTo(ParallelStitcher.GlobalPhase.FAILED_PERMANENT));

        System.out.flush();
        System.err.flush();
    }


    public StitchingReport getStitchingReport() {
        return parallelStitcher.getStitchingReport();
    }

    public ParallelStitcher getParallelStitcher() {
        return parallelStitcher;
    }

    public boolean isGaveUpOnStitching() {
        return gaveUpOnStitching;
    }

    public ApiCallDetailsCache getApiCallDetailsCache() {
        return cache;
    }
}
