package be.iminds.ilabt.jfed.lowlevel.connection;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.util.JFedPasswordManager;
import be.iminds.ilabt.jfed.util.JFedTrustStore;

import java.net.MalformedURLException;
import java.net.URL;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.List;

/**
 * ConnectionFactory
 */
public interface ConnectionBuilder {
    public void setUrl(String serverUrlString) throws MalformedURLException;
    public void setUrl(URL serverUrl);

    public void setDebugInfo(JFedConnection.DebugInfo debugInfo);

    public void setProxy(JFedConnection.ProxyInfo proxyInfo);

    public void useNoAuthentication();
    public void useHttpBasicAuthentication(JFedPasswordManager.LoginInfo login);
    public void useSslClientAuthentication(List<X509Certificate> clientCertChain, PrivateKey privateKey);

    public void useHttp();
    public void useHttps(final JFedTrustStore conTrustStore, HandleUntrustedCallback handleUntrustedCallback);

    public SfaConnection buildSfaConnection() throws JFedException;
    public HttpConnection buildHttpConnection() throws JFedException;

    public ConnectionConfig getCurrentConnectionConfig(ConnectionConfig.Type type);
    public void set(ConnectionConfig connectionConfig);

    public void setHackAllowAllServerCertificates(boolean hackAllowAllServerCertificates);
}

