package be.iminds.ilabt.jfed.lowlevel.ssh_key_info;

import be.iminds.ilabt.jfed.util.KeyUtil;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.TmpContentFile;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.TmpFile;
import org.slf4j.LoggerFactory;

import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.interfaces.RSAPrivateKey;

/**
 * BasicSshKeyInfo: SshKeyInfo with keys given in constructor
 * <p/>
 * typically just used to get keys, not typically used for files.
 * <p/>
 * does not support PuTTY
 */
public class BasicSshKeyInfo extends AbstractSshKeyInfo {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(BasicSshKeyInfo.class);

    private final PublicKey sshPublicKey;
    private final PrivateKey sshPrivateKey;

    public BasicSshKeyInfo(PublicKey sshPublicKey, PrivateKey sshPrivateKey) {
        this.sshPublicKey = sshPublicKey;
        this.sshPrivateKey = sshPrivateKey;
    }

    /**
     * may return null if not possible
     */
    public static BasicSshKeyInfo createFromPrivateKeyOnly(PrivateKey sshPrivateKey) {
        if (sshPrivateKey instanceof RSAPrivateKey) {
            PublicKey pubKey = KeyUtil.rsaPrivateKeyToRsaPublicKey((RSAPrivateKey) sshPrivateKey);
            BasicSshKeyInfo res = new BasicSshKeyInfo(pubKey, sshPrivateKey);
            return res;
        } else
            return null;
    }

    @Override
    public TmpFile getPrivateKeyFile() {
        return getUnencryptedPrivateKeyFile();
    }

    @Override
    public TmpFile getUnencryptedPrivateKeyFile() {
        String pemKey = new String(KeyUtil.privateKeyToAnyPem(sshPrivateKey));
        TmpContentFile unlockedPrivateKeyFile = new TmpContentFile("sskKeyBasUE", "pem", pemKey, true/*assureUserOnlyPermissions*/);
        ensureMatchingPubFile(unlockedPrivateKeyFile);
        return unlockedPrivateKeyFile;
    }


    @Override
    public TmpFile getPuttyKeyFile() {
        throw new UnsupportedOperationException("PuTTY files not supported here");
    }

    @Override
    public TmpFile getUnlockedPuttyKeyFile() {
        throw new UnsupportedOperationException("PuTTY files not supported here");
    }

    @Override
    public boolean hasUnencryptedPrivateKey() {
        return sshPrivateKey != null;
    }

//    @Override
//    public void release() {
//        for (File matchingPubKeyfile : matchingPubKeyfiles) {
//            matchingPubKeyfile.delete();
//        }
//        matchingPubKeyfiles.clear();
//    }

    @Override
    public PublicKey getPublicKey() {
        return sshPublicKey;
    }

    @Override
    public PrivateKey getPrivateKey() {
        return sshPrivateKey;
    }
}
