package be.iminds.ilabt.jfed.preferences;

import be.iminds.ilabt.jfed.util.OSDetector;

/**
* CorePreferenceKey
*/
public enum CorePreferenceKey implements JFedPreferences.PreferenceKey {
    PREF_DEBUG_CPU("debug.cpu"), //periodic dumping of CPU usage to logs or not (default false)   (should be passed to SfaConnectionPool)

    PREF_LOGGING_OVERWRITE("logging.overwrite"), //should jFed settings overwrite default settings? (if false, jFed will not try to change logging settings)
    PREF_LOGGING_ROOT_LEVEL("logging.logger.root_level"), //root log level
    PREF_LOGGING_EXTTERM_LEVEL("logging.logger.terminal_level"), //log level for starting external terminals
//    PREF_LOGGING_APPENDER_STDOUT_LEVEL("logging.appender.stdout_level"),
//    PREF_LOGGING_APPENDER_BUGREPORT_LEVEL("logging.appender.bugreport_level"),

    RETRY_TIMEOUT_MAX("retry.timeoutMaxRetries"),
    RETRY_TIMEOUT_MS_DELAY("retry.timeoutMsBeforeRetry"),
    RETRY_BUSY_MAX("retry.busyMaxRetries"),
    RETRY_BUSY_MS_DELAY("retry.busyMsBeforeRetry"),
    RETRY_POSTEMPERR_MAX("retry.possibleTemporaryErrorMaxRetries"),
    RETRY_POSTEMPERR_MS_DELAY("retry.possibleTemporaryErrorMsBeforeRetry"),

    PREF_TESTBEDS_XML_SOURCE("testbed.testbeds_xml.source"),  //"AUTO" (= use testbeds.xml or fallback), "file:*" or "http*" or "BUILTIN"

    PREF_TESTBEDSPECIFIC_EXOGENI_FORCE_EXOSM("testbed.exogeni.force_exo_sm"),  //AUTO (= use testbeds.xml), YES or NO
    PREF_TESTBEDSPECIFIC_EXOGENI_RECOVER_METHOD("testbed.exogeni.recover_method");  //BOTH or SINGLE:
                                                                                    //BOTH (= first use exoSM, fall back to others if nothing found there),
                                                                                    //SINGLE (= use what testbed.exogeni.force_exo_sm says)

    final String key;
    final OSDetector.OSFilter osFilter;

    CorePreferenceKey(String key) {
        this.key = key;
        this.osFilter = null;
    }

    CorePreferenceKey(String key, OSDetector.OSFilter osFilter) {
        this.key = key;
        this.osFilter = osFilter;
    }

    @Override
    public String toString() {
        return key;
    }

    @Override
    public String getKey() {
        return key;
    }

    @Override
    public boolean isOSSpecific() {
        return osFilter != null;
    }

    @Override
    public OSDetector.OSFilter getOSFilter() {
        return osFilter;
    }
}
