package be.iminds.ilabt.jfed.util;

import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;

/**
 * TargetAuthority: An authority to test or do something else with.
 *
 * This is simple 3 parts:
 *   - The original provided URN
 *   - The SfaAuthority derived from it that is used for connections
 *   - The SfaAuthority derived from it that is used as component_manager_id when generating RSpec's for it
 */
public class TargetAuthority {
    protected final GeniUrn origUrn;
    protected final GeniUrn rspecUrn;
    protected final SfaAuthority sfaAuthorityForRspec;
    protected final SfaAuthority sfaAuthorityToConnect;

    public TargetAuthority(GeniUrn origUrn, SfaAuthority sfaAuthorityForRspec, SfaAuthority sfaAuthorityToConnect) {
        this.origUrn = origUrn;
        this.sfaAuthorityForRspec = sfaAuthorityForRspec;
        this.rspecUrn = sfaAuthorityForRspec.getUrn();
        this.sfaAuthorityToConnect = sfaAuthorityToConnect;
    }
//    public TargetAuthority(GeniUrn origUrn, GeniUrn rspecUrn, SfaAuthority sfaAuthorityToConnect) {
//        this.origUrn = origUrn;
//        this.rspecUrn = rspecUrn;
//        this.sfaAuthorityToConnect = sfaAuthorityToConnect;
//    }

    public GeniUrn getOrigUrn() {
        return origUrn;
    }

    public GeniUrn getRspecUrn() {
        return rspecUrn;
    }

    public SfaAuthority getSfaAuthorityForRspec() {
        return sfaAuthorityForRspec;
    }

    public SfaAuthority getSfaAuthorityToConnect() {
        return sfaAuthorityToConnect;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof TargetAuthority)) return false;

        TargetAuthority that = (TargetAuthority) o;

        if (!origUrn.equals(that.origUrn)) return false;
        if (!rspecUrn.equals(that.rspecUrn)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = origUrn.hashCode();
        result = 31 * result + rspecUrn.hashCode();
        return result;
    }
}
