package be.iminds.ilabt.jfed;

import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.TestCertHelper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.name.Named;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.List;

/**
 * TestUserModule
 */
public class TestUserModule extends AbstractModule {
    private final static Logger LOG = LoggerFactory.getLogger(TestUserModule.class);

    public TestUserModule() {
    }

    @Override
    protected void configure() {
    }

    @Provides
    public GeniUser provideGeniUser(final AuthorityListModel authorityListModel, final TestCertHelper testCertHelper) {
        return new GeniUser() {
            @Override
            public PrivateKey getPrivateKey() {
                assert testCertHelper != null;
                assert testCertHelper.userKeys != null;
                return testCertHelper.userKeys.getSshPrivateKey();
            }

            @Override
            public PublicKey getPublicKey() {
                return testCertHelper.userKeys.getSshPublicKey();
            }

            @Override
            public List<X509Certificate> getClientCertificateChain() {
                return Arrays.asList(testCertHelper.userCert);
            }

            @Override
            public SfaAuthority getUserAuthority() {
                return authorityListModel.getFromAnyUrnExact(TestCertHelper.authority1Urn);
            }

            @Override
            public String getUserUrnString() {
                return TestCertHelper.userUrn;
            }

            @Override
            public GeniUrn getUserUrn() {
                return GeniUrn.parse(getUserUrnString());
            }

            @Override
            public File getPrivateKeyFile() {
                return null;
            }

            @Override
            public File getCertificateFile() {
                return null;
            }
        };
    }

    @Provides
    @Named("userAuthority")
    public SfaAuthority provideUserAuth(final GeniUser geniUser) {
        return geniUser.getUserAuthority();
    }

    @Provides
    public GeniUserProvider provideGeniUserProvider(final GeniUser geniUser) {
        return new GeniUserProvider() {
            @Override
            public GeniUser getLoggedInGeniUser() {
                return geniUser;
            }

            @Override
            public boolean isUserLoggedIn() {
                return geniUser != null;
            }
        };
    }
}
